# Create your views here.
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404
# Imaginary function to handle an uploaded file.
#from somewhere import handle_uploaded_file
from django.core.paginator import Paginator
from .models import Dbnfsp, Genome1kVariant, Genome1kSample, Genome1kGenotype, Genome1kSampleVariant, Genome1kVariantIndex
from .forms import Genomes1kForm
from datetime import datetime


@login_required
def index(request):

    return render(request, 'databases/index.html')


def dbnfsp_index(request):

    dbnfsp_list = Dbnfsp.objects.first()
    # paginator = Paginator(dbnfsp_list, 25)

    context = {'dbnfsp_list':dbnfsp_list}

    return render(request, 'dbnfsp/index.html', context)    

def genomes1k_index(request):

    start = datetime.now()
    if request.method == 'POST':
        # create a form instance and populate it with data from the request:
        form = Genomes1kForm(request.POST)
        if form.is_valid():
            sample = form.cleaned_data['sample']
            genomes1k_variants = Genome1kSampleVariant.objects.filter(sample__name=sample, genotype__genotype='1|1')
            query_variants = genomes1k_variants.count()
            # query_variants = 0
            # len(genomes1k_variants)

    # if a GET (or any other method) we'll create a blank form
    else:
        form = Genomes1kForm()
        genomes1k_variants = Genome1kSampleVariant.objects.all()[:10]
        query_variants = 0#genomes1k_variants.count()
    
    total_variants = Genome1kSampleVariant.objects.all().count()
    # total_variants = 0

    # for item in genomes1k_variants:
    #   print(dir(item))
    # paginator = Paginator(dbnfsp_list, 25)

    end = datetime.now()
    time_taken = end-start


    context = {
    'genomes1k_variants':genomes1k_variants,
    'total_variants':total_variants,
    'query_variants':query_variants,
    'time_taken':time_taken,
    'form':form
    }


    return render(request, '1000genomes/index.html', context)   


from django.shortcuts import render
from django.shortcuts import redirect
from django.template import RequestContext
from .forms import CaseForm
from cases.models import Case
from django.contrib.auth.decorators import login_required

from django.shortcuts import render
from django.shortcuts import get_object_or_404

from django.views.generic import CreateView, DeleteView

from individuals.models import Individual
from django.contrib import messages
from django.urls import reverse

from django.contrib.auth.mixins import LoginRequiredMixin


# Create your views here.
@login_required
def cases_list(request):

    if request.user.is_staff:
        cases = Case.objects.all().order_by('id')
    else:
        cases = Case.objects.filter(user=request.user).order_by('id')

    context = {'cases': cases}

    return render(request, 'cases/list.html', context)


@login_required
def create_case(request):
    if request.method == 'POST':
        form = CaseForm(request.POST, request.FILES)        
        if form.is_valid():            
            case = form.save(commit=False)
            case.user = request.user
            case.save()
            form.save_m2m()
            return redirect('cases_list')
    else:
        form = CaseForm()
    return render(request, 'cases/new.html', {'form': form})

@login_required
def view_case(request, case_id):

    if request.user.is_staff:
        case = get_object_or_404(Case, pk=case_id)
    else:
        case = get_object_or_404(Case, pk=case_id, user=request.user)

    return render(request, 'cases/view.html', {'case': case})


class CaseDeleteView(LoginRequiredMixin, DeleteView):

    model = Case

    def delete(self, request, *args, **kwargs):
        """
        This does not actually delete the file, only the database record.  But
        that is easy to implement.
        """
        self.object = self.get_object()
        
        self.object.delete()
        messages.add_message(request, messages.INFO, "Case deleted with success!")
        return redirect('cases_list')
    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects


    
@login_required
def edit(request, case_id):
    
    if request.user.is_staff:
        case = get_object_or_404(Case, pk=case_id)
    else:
        case = get_object_or_404(Case, pk=case_id, user=request.user)

        
    if request.method == 'POST':
        form = CaseForm(request.POST, request.FILES, instance=case)        
        if form.is_valid():
            form.save()
#            variants = form.cleaned_data['variants']
#            strs = form.cleaned_data['strs']
#            cnvs = form.cleaned_data['cnvs']
#            #use id for unique names
#            individual = Individual.objects.create(user=request.user, status='new')
#            
#            individual.variants=variants
#            individual.strs=cnvs
#            individual.cnvs=cnvs
#            
#            individual.name=request.POST['name']
#            individual.save()
#            AnnotateVariants.delay(individual.id)
            return redirect('cases_list')
    else:
        form = CaseForm(instance=case)
    return render(request, 'cases/edit.html', {'form': form})

@login_required
def analysis(request, case_id, analysis, inheritance):
    
    if request.user.is_staff:
        case = get_object_or_404(Case, pk=case_id)
    else:
        case = get_object_or_404(Case, pk=case_id, user=request.user)
        
    query_string = []
    query_string.append('variants_per_gene=')

    query_string.append('impact=HIGH')
    query_string.append('impact=MODERATE')
    query_string.append('dbsnp_option=>')
    query_string.append('dbsnp_build=130')
    query_string.append('read_depth_option=>')
    query_string.append('read_depth=10')
    # query_string.append('')
    query_string.append('genomes1000=0+-+0.005')
    query_string.append('dbsnp_frequency=0+-+0.005')
    query_string.append('esp_frequency=0+-+0.005')

    
    if analysis == 'filter_analysis':
        query_string.append('genes_in_common=on')

    if analysis == 'filter_analysis' or analysis == 'pathway_analysis' :

        #add children and cases to individuals
        for individual in case.children.all():
            query_string.append('individuals=%s' % (individual.id))
        for individual in case.cases.all():
            query_string.append('individuals=%s' % (individual.id))
        for individual in case.controls.all():
            query_string.append('exclude_individuals=%s' % (individual.id))

        #add father and mother to exclude individuals
        if case.father:
            query_string.append('exclude_individuals=%s' % (case.father.id))
        if case.mother:
            query_string.append('exclude_individuals=%s' % (case.mother.id))
        
        
        # filterstring = "&".join(query_string)
        # return redirect(reverse('filter_analysis')+'?'+filterstring)

    if analysis == 'family_analysis':
        query_string.append('genes_in_common=on')

        query_string.append('remove_not_in_parents=on')

        if case.father:
            query_string.append('father=%s' % (case.father.id))
        if case.mother:
            query_string.append('mother=%s' % (case.mother.id))
        for individual in case.children.all():
            query_string.append('children=%s' % (individual.id))
        for individual in case.cases.all():
            query_string.append('individuals=%s' % (individual.id))
        for individual in case.controls.all():
            query_string.append('exclude_individuals=%s' % (individual.id))



        if inheritance == 'recessive_homozygous':
            query_string.append('inheritance_option=1')
        if inheritance == 'dominant_heterozygous':
            query_string.append('inheritance_option=2')
        if inheritance == 'compound_heterozygous':
            query_string.append('inheritance_option=3')
        if inheritance == 'x_linked':
            query_string.append('chr=X')
            query_string.append('inheritance_option=2')

    #inheritance
    if inheritance == 'recessive_homozygous':        
        query_string.append('mutation_type=homozygous')
    if inheritance == 'compound_heterozygous':        
        query_string.append('mutation_type=heterozygous')
        query_string.append('variants_per_gene_option=>')
        query_string.append('variants_per_gene=2')
    if inheritance == 'dominant_heterozygous':        
        query_string.append('mutation_type=heterozygous')
    if inheritance == 'x_linked':        
        query_string.append('mutation_type=homozygous')
        query_string.append('chr=X')


    #join all options
    filterstring = "&".join(query_string)

    if analysis == 'filter_analysis':
        url_redirect = 'filter_analysis'
    if analysis == 'family_analysis':
        url_redirect = 'family_analysis'
    if analysis == 'pathway_analysis':
        url_redirect = 'pathway_filter_analysis'
    
    return redirect(reverse(url_redirect)+'?'+filterstring)


from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^$', views.index, name='settings-index'),    
    url(r'^s3-create/$', views.create_s3_credential, name='settings-s3-create'),
    url(r'^s3-edit/(?P<pk>[0-9]+)/$', views.S3CredentialUpdate.as_view(), name='settings-s3-update'),
    url(r'^s3-view/(?P<pk>[0-9]+)/$', views.S3CredentialDetailView.as_view(), name='settings-s3-detail'),    
    url(r'^s3-delete/(?P<pk>[0-9]+)/$', views.S3CredentialDelete.as_view(), name='settings-s3-delete'),
]



import gzip
import json
import os

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.paginator import EmptyPage
from django.core.paginator import PageNotAnInteger
from django.core.paginator import Paginator
from django.db.models import Avg
from django.db.models import Count
from django.db.models import Max
from django.db.models import Min
from django.db.models import Q
from django.db.models import Sum
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.shortcuts import render
from django.template import RequestContext
from django.utils.text import slugify
from django.views.generic import DeleteView
from individuals.forms import IndividualForm, ComparisonForm, GroupForm, BrowserForm
from individuals.models import Individual, Group
from individuals.tasks import VerifyVCF, AnnotateVariants, PopulateVariants
from variants.models import Variant

def response_mimetype(request):
    if "application/json" in request.META['HTTP_ACCEPT']:
        return "application/json"
    else:
        return "text/plain"


class JSONResponse(HttpResponse):
    """JSON response class."""
    def __init__(self,obj='',json_opts={},mimetype="application/json",*args,**kwargs):
        content = json.dumps(obj,**json_opts)
        super(JSONResponse,self).__init__(content,mimetype,*args,**kwargs)

def create(request):
    if request.method == 'POST':
        form = IndividualForm(request.POST, request.FILES)
        
        if form.is_valid():
            
            if request.user.is_authenticated:
                individual = Individual.objects.create(user=request.user, status='new')
            else:
                individual = Individual.objects.create(user=None, status='new')

            individual.vcf_file= request.FILES.get('file')
            
            print('file')
            print(request.FILES.get('file'))

            filename = individual.vcf_file.name.split('.')
            new_filename = [] 
            for tag in filename:
                new_filename.append(slugify(tag))

            individual.vcf_file.name = ".".join(new_filename)

            print('filename ', filename)

            #get name from inside vcf file
            individual.name= str(os.path.splitext(individual.vcf_file.name)[0]).replace('.vcf','').replace('.gz','').replace('.rar','').replace('.zip','').replace('._',' ').replace('.',' ')

            # individual.shared_with_groups = form.cleaned_data['shared_with_groups']

            individual.shared_with_groups.set(form.cleaned_data['shared_with_groups'])
            
            individual.save()
            
            f = individual.vcf_file
            
            #fix permissions
            #os.chmod("%s/genomes/%s/" % (settings.BASE_DIR, individual.user), 0777)

            #if request.user.is_authenticated:

            #    os.chmod("%s/genomes/%s/%s" % (settings.BASE_DIR, slugify(individual.user), individual.id), 0o777)
            #else:
            #    os.chmod("%s/genomes/public/%s" % (settings.BASE_DIR, individual.id), 0o777)

            # AnnotateVariants.delay(individual.id)
            VerifyVCF.delay(individual.id)

            data = {'files': [{'deleteType': 'DELETE', 'name': individual.name, 'url': '', 'thumbnailUrl': '', 'type': 'image/png', 'deleteUrl': '', 'size': f.size}]}

            response = JSONResponse(data, mimetype=response_mimetype(request))
            response['Content-Disposition'] = 'inline; filename=files.json'

            return response
        else:
            print(form.errors)

    else:
        form = IndividualForm()

    return render(request, 'individuals/create.html', {'form':form})

# Create your views here.
@login_required
def edit(request, individual_id):
    individual = get_object_or_404(Individual, pk=individual_id)
    if request.method == 'POST':
        form = IndividualForm(request.POST, instance=individual)
        if form.is_valid():
            form.save()
            return redirect('dashboard')
    #     form = IndividualForm(request.POST, request.FILES)
    #     if form.is_valid():
    #         individual = form.save(commit=False)
    #         individual.user = request.user
    #         individual.save()
    #         return redirect('dashboard')
    else:
        form = IndividualForm(instance=individual)

    return render(request, 'individuals/individual_form.html', {'form':form})


class IndividualDeleteView(DeleteView):
    model = Individual

    def delete(self, request, *args, **kwargs):
        """
        This does not actually delete the file, only the database record.  But
        that is easy to implement.
        """
        self.object = self.get_object()
        individual_id = self.object.id

        if self.object.user:
            username = self.object.user.username
        else:
            username = 'public'
        
        #delete files
        if self.object.vcf_file:
            self.object.vcf_file.delete()

        # if self.object.strs_file:
        #     self.object.strs_file.delete()
        # if self.object.cnvs_file:
        #     self.object.cnvs_file.delete()
        os.system('rm -rf %s/genomes/%s/%s' % (settings.BASE_DIR, username, individual_id))

        self.object.delete()
        
        
        
        
#        response = JSONResponse(True, {}, response_mimetype(self.request))
#        response['Content-Disposition'] = 'inline; filename=files.json'
#        return response
        messages.add_message(request, messages.INFO, "Individual deleted with success!")
        #return redirect('individuals_list')
        return redirect('individuals_list')


def view(request, individual_id):

    individual = get_object_or_404(Individual, pk=individual_id)

    variant_list = Variant.objects.filter(individual=individual)
    # snpeff = SnpeffAnnotation.objects.filter(individual=individual)

    individual.n_variants = variant_list.count()
    individual.novel_variants = variant_list.filter(variant_id = '.').count()

    individual.summary = []

    #get calculated values from database

    summary_item = {
                'type': 'Total SNVs',
                'total': variant_list.values('genotype').count(),
                'discrete': variant_list.values('genotype').annotate(total=Count('genotype'))
                    }
    individual.summary.append(summary_item)

    summary_item = {
                'type': 'Total Gene-associated SNVs',
                'total': variant_list.values('gene').exclude(gene="").count(),
                'discrete': variant_list.exclude(gene="").values('genotype').annotate(total=Count('genotype'))
                    }
    individual.summary.append(summary_item)

    individual.snp_eff = variant_list.values('snpeff_effect').annotate(Count('snpeff_effect')).order_by('snpeff_effect')
    # print 'individual.snp_eff', individual.snp_eff
    # variant_list.values('snpeff__effect').annotate(Count('snpeff__effect')).order_by('snpeff__effect')
    #
    individual.functional_class = variant_list.values('snpeff_func_class').annotate(Count('snpeff_func_class')).order_by('snpeff_func_class')
    individual.impact_variants = variant_list.values('snpeff_impact').annotate(Count('snpeff_impact')).order_by('snpeff_impact')

    individual.filter_variants = variant_list.values('filter').annotate(Count('filter')).order_by('filter')
    individual.quality = variant_list.aggregate(Avg('qual'), Max('qual'), Min('qual'))
    individual.read_depth = variant_list.aggregate(Avg('read_depth'), Max('read_depth'), Min('read_depth'))

    individual.clinvar_clnsig = variant_list.values('clinvar_clnsig').annotate(total=Count('clinvar_clnsig'))

    individual.chromossome = variant_list.values('chr').annotate(total=Count('chr')).order_by('chr')

    # variants_with_snpid = variant_list.values('variant_id').exclude(variant_id=".")
    #print variants_with_snpid

    # fields = Variant._meta.get_all_field_names()

    paginator = Paginator(variant_list, 25) # Show 25 contacts per page
    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1
    try:
        variants = paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        variants = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        variants = paginator.page(paginator.num_pages)
    #'fields':fields
    return render(request, 'individuals/view.html', {'individual': individual, 'variants':variants})

@login_required
def browse(request, individual_id):

    query_string = request.META['QUERY_STRING']
    individual = get_object_or_404(Individual, pk=individual_id)
    query = {}
    
#    DEFAULT_SORT = 'pk'
#    sort_key = request.GET.get('sort', DEFAULT_SORT)
    # tags = ['genotype', 'snpeffannotation__effect']#, 'func_class', 'impact', 'cln_omim', 'chr' 
    # for tag in tags:
    #     criteria = request.GET.get(tag, '')
    #     if criteria:
    #         query[tag] = criteria

    
    if request.method == 'GET':
        form = BrowserForm(request.GET)   
        if form.is_valid():
            print('form is valid')
            #chr
            chr = request.GET.get('chr', '')
            if chr != '':
                query['chr'] = chr
            #pos
            pos = request.GET.get('pos', '')
            if pos != '':
                query['pos'] = pos

            effect = request.GET.get('effect', '')
            if effect != '':
                print('effect', effect)
                query['snpeff_effect'] = effect                
            #snp_id
            # snp_id = request.GET.get('snp_id', '')
            # if snp_id != '':
            #     query['variant_id'] = snp_id
            # snp_list = request.GET.get('snp_list', '')
            # snp_list = snp_list.split('\r\n')
            # if snp_list[0] != u'':
            #     query['variant_id__in'] = snp_list
            # snp_eff = request.GET.getlist('effect')
            # if len(snp_eff) > 0:
            #     query['snp_eff__in'] = snp_eff
            # func_class = request.GET.getlist('func_class')
            # if len(func_class) > 0:
            #     query['snp_eff_functional_class__in'] = func_class
            # gene = request.GET.get('gene', '')
            # if gene != '':
            #     query['gene_name'] = gene
            # gene_list = request.GET.get('gene_list', '')
            # gene_list = gene_list.split('\r\n')
            # if gene_list[0] != u'':
            #     query['gene_name__in'] = gene_list
            # cln = request.GET.get('cln_omim', '')
            # print 'clnomim', cln
            # if cln == 'on':
            #     query['cln_omim'] != ''

            variants = Variant.objects.filter(individual=individual, **query)

            # snpeff_annotations = SnpeffAnnotation.objects.filter(variant__in=variants)
            # #b.entry_set.filter(headline__contains='Lennon')

            # print 'snpeff_annotations', len(snpeff_annotations)

            # for variant in variants:
            #     print variant.entry_set.all()
            
            #     variant.snpeff=

    
    else:
        
        form = BrowserForm(request.GET)
        variants = Variant.objects.filter(individual=individual, **query)
        
        
    #Pagination
    paginator = Paginator(variants, 25) # Show 25 contacts per page
    try:
       page = int(request.GET.get('page', '1'))
    except ValueError:
       page = 1
    try:
       variants = paginator.page(page)
    except PageNotAnInteger:
       # If page is not an integer, deliver first page.
       variants = paginator.page(1)
    except EmptyPage:
       # If page is out of range (e.g. 9999), deliver last page of results.
       variants = paginator.page(paginator.num_pages)
    
    return render(request, 'variants/variants.html', {'individual': individual, 'variants':variants, 'form':form, 'query_string':query_string})

@login_required
def list(request):

    if request.method == 'POST':
        individuals = request.POST.getlist('individuals')
        print(individuals) 
        individuals = [int(x) for x in individuals]
        print(individuals)
        
        if request.POST['selectionField'] == "Show":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                individual.is_featured = True
                individual.save()
        if request.POST['selectionField'] == "Hide":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                individual.is_featured = False
                individual.save()
        if request.POST['selectionField'] == "Delete":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)


                individual_id = individual.id
                username = individual.user.username

                #delete files
                if individual.vcf_file:
                    individual.vcf_file.delete()
                # if individual.strs_file:
                #     individual.strs_file.delete()
                # if individual.cnvs_file:
                #     individual.cnvs_file.delete()
                os.system('rm -rf %s/genomes/%s/%s' % (settings.BASE_DIR, username, individual_id))

                individual.delete()
            #os.system('rm -rf mendelmd14/site_media/media/genomes/%s/%s' % (username, individual_id))
        if request.POST['selectionField'] == "Populate":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                PopulateVariants.delay(individual.id)
            
        if request.POST['selectionField'] == "Annotate":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                AnnotateVariants.delay(individual.id)
        if request.POST['selectionField'] == "Find_Medical_Conditions_and_Medicines":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                Find_Medical_Conditions_and_Medicines.delay(individual.id)
        
    
    args = []
    # groups = Groups.objects.filter(user=request.user, shared_with_users=).order_by("-id")
    args.append(Q(user=request.user) | Q(shared_with_users=request.user) | Q(shared_with_groups__members=request.user))
    
    if request.user.is_staff:
        individuals = Individual.objects.all()
    else:
        individuals = Individual.objects.filter(*args).order_by("-id")
    
    ind_featured = Individual.objects.filter(is_featured= True).order_by("id")
    # paginator = Paginator(individuals, 25) # Show 25 contacts per page
           
    # try:
    #    page = int(request.GET.get('page', '1'))
    # except ValueError:
    #    page = 1
    # try:
    #    individuals = paginator.page(page)
    # except PageNotAnInteger:
    #    # If page is not an integer, deliver first page.
    #    individuals = paginator.page(1)
    # except EmptyPage:
    #    # If page is out of range (e.g. 9999), deliver last page of results.
    #    individuals = paginator.page(paginator.num_pages)



    groups = Group.objects.all()
#    individuals = Individual.objects.annotate(number_of_variants=Count('variant'))
    
    
    return render(request, 'individuals/list.html', {'individuals': individuals, 'groups':groups, 'ind_featured':ind_featured})


@login_required
def annotate(request, individual_id):
    individual = get_object_or_404(Individual, pk=individual_id)
    individual.status = 'new'
    individual.n_lines = 0
    VerifyVCF.delay(individual.id)
    individual.save()
    messages.add_message(request, messages.INFO, "Your individual is being annotated.")
    return redirect('dashboard')

@login_required
def populate(request, individual_id):
    individual = get_object_or_404(Individual, pk=individual_id)
    PopulateVariants.delay(individual.id)
    messages.add_message(request, messages.INFO, "Your individual is being populated.")

    return redirect('dashboard')


@login_required
def populate_mongo(request, individual_id):
    individual = get_object_or_404(Individual, pk=individual_id)
    PopulateMongoVariants.delay(individual.id)
    messages.add_message(request, messages.INFO, "Your individual is being inserted at MongoDB.")

    return redirect('individuals_list')


def download(request, individual_id):
    individual = get_object_or_404(Individual, pk=individual_id)
    
    filepath = os.path.dirname(str(individual.vcf_file.name))
    filename = os.path.basename(str(individual.vcf_file.name))
    
    path = ''
    # os.chmod("%s/genomes/%s/%s" % (settings.MEDIA_ROOT, individual.user, individual.id), 0777)

    
    # if filename.endswith('vcf.zip'):
       # basename = filename.split('.vcf.zip')[0]       
    # elif filename.endswith('.zip'):
       # basename = filename.split('.zip')[0]       
    # else:
       # basename = filename.split('.vcf')[0]
    #print basename
    #print path
    #print filepath
    
    fullpath = '%s/%s' % (filepath, filename)
    if filename.endswith('.gz'):
        vcffile = gzip.open(fullpath, 'r')
    else:
        vcffile = open(fullpath, 'r')

    content = vcffile.read()
    vcffile.close()

    response = HttpResponse(content, content_type='text/plain')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    response['Content-Length'] = os.path.getsize(fullpath)
    return response



def download_annotated(request, individual_id):
    individual = get_object_or_404(Individual, pk=individual_id)
    
    filepath = os.path.dirname(str(individual.vcf_file.name))
    filename = os.path.basename(str(individual.vcf_file.name))
    
    # path = settings.MEDIA_ROOT
    # if filename.endswith('vcf.zip'):
       # basename = filename.split('.vcf.zip')[0]       
    # else:
    
    basename = filename.split('.vcf')[0]
    
    fullpath = '%s/annotation.final.vcf.zip' % (filepath)

    vcffile = open(fullpath, 'rb')

    response = HttpResponse(vcffile, content_type='application/x-zip-compressed')
    # # response['Content-Encoding'] = 'gzip'
    response['Content-Disposition'] = 'attachment; filename=%s.annotated.mendelmd.vcf.zip' % basename
    response['Content-Length'] = os.path.getsize(fullpath)
    return response

@login_required
def create_group(request):
    if request.method == 'POST':
        form = GroupForm(request.POST, request.FILES)        
        if form.is_valid():
            form.save()
            
            return redirect('individuals_list')
    else:
        form = GroupForm()
    return render(request, 'groups/create_group.html', {'form': form})

@login_required
def view_group(request, group_id):
    group = get_object_or_404(Group, pk=group_id)
    return render(request, 'groups/view_group.html', {'group': group})

class GroupDeleteView(DeleteView):
    model = Group

    def delete(self, request, *args, **kwargs):
        """
        This does not actually delete the file, only the database record.  But
        that is easy to implement.
        """
        self.object = self.get_object()
        
        #username = self.object.user.username
        
        self.object.delete()
        messages.add_message(request, messages.INFO, "Group deleted with success!")
        return redirect('individuals_list')


def comparison(request):
    query = {}
    summary = {}
    variants = []
    query_string = request.META['QUERY_STRING']
    if request.method == 'GET':
        form = ComparisonForm(request.user, request.GET, request.FILES)        
        if form.is_valid():
            
            individual_one_id = request.GET.get('individual_one', '')
            individual_two_id = request.GET.get('individual_two', '')
            read_depth = request.GET.get('read_depth', '')
            if read_depth != '':
                query['read_depth__gte'] = float(read_depth)
            if individual_one_id != '' and individual_two_id != '': 
                variants_ind_one = Variant.objects.filter(individual__id=individual_one_id, **query).values('chr', 'pos', 'genotype')
                
                variants_ind_two = Variant.objects.filter(individual__id=individual_two_id, **query).values('chr', 'pos', 'genotype')
                print('Got Variants from Both!')
    
                genotypes_in_common = 0
                genotypes_not_in_common = 0
                ind_one = {}
                ind_two = {}
                summary['variants_ind_one'] = variants_ind_one.count()
                 
                for variant in variants_ind_one:
                    id = '%s-%s' % (variant['chr'], variant['pos'])
                    if id in ind_one:                 
                        ind_one[id].append(variant['genotype'])
                    else:
                        ind_one[id] = []
                        ind_one[id].append(variant['genotype'])
                        
                summary['variants_ind_two'] = variants_ind_two.count()
                for variant in variants_ind_two:
                    id = '%s-%s' % (variant['chr'], variant['pos'])
                    if id in ind_two:                 
                        ind_two[id].append(variant['genotype'])
                    else:
                        ind_two[id] = []
                        ind_two[id].append(variant['genotype'])
                    
                
                print('Finished creating indexes')
                for pos in ind_one:
                    if pos in ind_two:
                        for genotype in ind_one[pos]:
                            
                            if genotype in ind_two[pos]:
                                genotypes_in_common += 1
    #                            variant ={}
    #                            variant['chr'] = item.split('-')[0]
    #                            variant['pos'] = item.split('-')[1]
    #                            variant['genotype'] = ind_two[item]
                                
    #                            variants.append(variant) 
                            else:
                                genotypes_not_in_common += 1
                        
    #                        
                print('genotypes in common: %s' % genotypes_in_common)
                summary['genotypes_in_common'] = genotypes_in_common
                summary['genotypes_not_in_common'] = genotypes_not_in_common
                summary['total_variants'] = genotypes_in_common + genotypes_not_in_common
                
                summary['percent_ind_one'] = round((float(genotypes_in_common)/summary['variants_ind_one'])*100, 2)
                summary['percent_ind_two'] = round((float(genotypes_in_common)/summary['variants_ind_two'])*100, 2)
                print(summary)
            
    else:
        form = ComparisonForm(request.user)
        
        
    return render(request, 'individuals/comparison.html', {'form':form, 'summary':summary, 'query_string':query_string})



from django.db import models

from django.urls import reverse

from projects.models import Project
from settings.models import Provider
from samples.models import Sample
from tasks.models import Task
from files.models import File
from mapps.models import App as Mapp

from django.contrib.auth.models import User, Group
from django.contrib.postgres.fields import JSONField

class AnalysisType(models.Model):
    class Meta:
        verbose_name_plural = "analysis_types"

    name = models.CharField(max_length=30)
    repository = models.CharField(max_length=600, null=True, blank=True)
    def __str__(self):
        return self.name


class Analysis(models.Model):
    class Meta:
        verbose_name_plural = "analyses"

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    params = JSONField(null=True, blank=True)

    name = models.CharField(max_length=30)
    status = models.TextField(null=True, blank=True)
    project = models.ForeignKey(Project, on_delete=models.CASCADE, null=True, blank=True)
    apps = models.ManyToManyField(Mapp)
    samples = models.ManyToManyField(Sample)
    tasks = models.ManyToManyField(Task)
    files = models.ManyToManyField(File)

    # type = models.ForeignKey(AnalysisType, on_delete=models.CASCADE)
    # provider = models.ForeignKey(Provider, on_delete=models.CASCADE)

    def get_absolute_url(self):
        return reverse('analysis-detail', kwargs={'pk': self.pk})

    def __str__(self):
        return self.name




import boto3
from time import sleep
from subprocess import run
from django.conf import settings

class AWS:

    def __init__(self):
        pass

    def main(self):
        pass

    def launch(self, config):
        print('Launch Worker')

        worker = {}

        session = boto3.Session(profile_name=config['profile_name'])

        ec2 = session.resource('ec2', region_name=config['region_name'])
        print('Create instance')
        client = session.client('ec2', region_name=config['region_name'])

        response = client.request_spot_instances(
            DryRun=False,
            SpotPrice=config['spot_price'],
            InstanceCount=1,
            Type='one-time',
            LaunchSpecification={
                'ImageId': config['image_id'],
                'KeyName': config['KeyName'],
                'SubnetId': config['SubnetId'],
                'InstanceType': config['instance_type'],
                'Placement': {
                    'AvailabilityZone': config['AvailabilityZone'],
                },
                'EbsOptimized': True,
                'SecurityGroupIds': [
                    config['SecurityGroupIds'],
                ],
                #'IamInstanceProfile': {
                #    'Name': config['IamInstanceProfile'],
                #}
            }
        )

        print(response)
        # this works!
        sir_id = response['SpotInstanceRequests'][0]['SpotInstanceRequestId']
        print(sir_id)

        fulfilled = False
        while not fulfilled:
            try:
                response = client.describe_spot_instance_requests(
                    DryRun=False,
                    SpotInstanceRequestIds=[
                        sir_id,
                    ]
                )
                # print(response)
                instance_id = response['SpotInstanceRequests'][0]['InstanceId']
                fulfilled = True
            except KeyError:
                print('Sleep 30...')
                sleep(30)
                pass

        print(instance_id)

        instance = ec2.Instance(instance_id)
        tag = instance.create_tags(
            Tags=[
                {
                    'Key': 'Name',
                    'Value': 'MendelMD Worker'
                }
            ]
        )

        instance.wait_until_running()
        print(instance.private_ip_address)
        worker = {
            'id': instance_id,
            'ip': instance.private_ip_address,
        }
        return(worker)

    def terminate(self, instance_id):
        print('Terminate Worker')
        ec2 = boto3.resource('ec2', region_name='eu-west-1')
        print('Create instance')
        client = boto3.client('ec2', region_name='eu-west-1')
        response = client.terminate_instances(
            InstanceIds=[instance_id,],
            DryRun=False
        )
        return(response)

    def install(self, ip):

        print('Install Worker')

        command = "scp -r -o StrictHostKeyChecking=no ~/.aws ubuntu@%s:~/" % (ip)
        run(command, shell=True)

        command = "scp -o StrictHostKeyChecking=no ~/.ssh/id_rsa ubuntu@%s:~/.ssh" % (ip)
        run(command, shell=True)

        command = "scp -o StrictHostKeyChecking=no ~/.ssh/id_rsa.pub ubuntu@%s:~/.ssh" % (ip)
        run(command, shell=True)

        command = "scp -o StrictHostKeyChecking=no ~/.ssh/config ubuntu@%s:~/.ssh" % (ip)
        run(command, shell=True)

        command = "scp -o StrictHostKeyChecking=no scripts/install_worker_aws.sh ubuntu@%s:~/" % (ip)
        run(command, shell=True)

        command = "scp -o StrictHostKeyChecking=no /projects/scripts/local_settings.py ubuntu@%s:~/" % (ip)
        run(command, shell=True)

        command = """nohup bash install_worker_aws.sh >nohup.out 2>&1 & sleep 2"""
        command = """ssh -o StrictHostKeyChecking=no -t ubuntu@%s '%s'""" % (ip, command)

        run(command, shell=True)

    def update(self,ip):

        command = "scp -o StrictHostKeyChecking=no scripts/update_worker_aws.sh ubuntu@%s:~/" % (ip)
        run(command, shell=True)

        command = """nohup bash update_worker_aws.sh >nohup.out 2>&1 & sleep 5"""
        command = """ssh -o StrictHostKeyChecking=no -t ubuntu@%s '%s'""" % (ip, command)
        
        print(command)

        run(command, shell=True)

    def upload(source,dest):

        dest = '{}{}'.format(settings.UPLOAD_FOLDER, dest)

        command = 'aws s3 sync --profile {} {} {}'.format(settings.UPLOAD_FOLDER_PROFILE, source, dest)
        run(command, shell=True)




if __name__ == '__main__':
    worker = AWS().main()


from django.db import models

# Create your models here.
class VariSNP(models.Model):
    """
    Description: 

    benchmark database for neutral variations from dbSNP


    """
    dbsnp_id = models.CharField(max_length=255)
    # heterozygosity
    # heterozygosity_standard_error
    # creation_date
    # creation_build
    # update_date
    # update_build
    # observed_alleles
    # asn_from
    # asn_to
    # reference_allele
    # orientation
    # minor_allele_frequency
    # minor_allele
    # sample_size
    # validation
    # hgvs_names
    # allele_origin
    # clinical_significance
    # functional_class
    # ncbi_gi
    # ncbi_accession
    # gene_symbol
    # refseq_start_description
    # coding_dna_description
    # protein_description
    # coding_reference
    # protein_reference
    # predicted_RNA_variation
    # DNA_annotation
    # RNA_annotation
    # protein_annotation    

class HGMD(models.Model):
    index = models.TextField(db_index=True)
    chrom = models.TextField(null=True, blank=True, db_index=True)
    pos = models.TextField(null=True, blank=True, db_index=True)
    rsid = models.TextField(null=True, blank=True)
    ref = models.TextField(null=True, blank=True, db_index=True)
    alt = models.TextField(null=True, blank=True, db_index=True)
    qual = models.TextField(null=True, blank=True)
    filter = models.TextField(null=True, blank=True)
    mutclass = models.TextField(null=True, blank=True)
    mut = models.TextField(null=True, blank=True)
    gene = models.TextField(null=True, blank=True)
    strand = models.TextField(null=True, blank=True)
    dna = models.TextField(null=True, blank=True)
    prot = models.TextField(null=True, blank=True)
    db = models.TextField(null=True, blank=True)
    phen = models.TextField(null=True, blank=True)


class Dbnfsp(models.Model):
    chr = models.TextField(null=True, blank=True)
    pos_1_based = models.TextField(null=True, blank=True)
    ref = models.TextField(null=True, blank=True)
    alt = models.TextField(null=True, blank=True)
    aaref = models.TextField(null=True, blank=True)
    aaalt = models.TextField(null=True, blank=True)
    rs_dbSNP150 = models.TextField(null=True, blank=True)
    hg19_chr = models.TextField(null=True, blank=True)
    hg19_pos_1_based = models.TextField(null=True, blank=True)
    hg18_chr = models.TextField(null=True, blank=True)
    hg18_pos_1_based = models.TextField(null=True, blank=True)
    genename = models.TextField(null=True, blank=True)
    cds_strand = models.TextField(null=True, blank=True)
    refcodon = models.TextField(null=True, blank=True)
    codonpos = models.TextField(null=True, blank=True)
    codon_degeneracy = models.TextField(null=True, blank=True)
    Ancestral_allele = models.TextField(null=True, blank=True)
    AltaiNeandertal = models.TextField(null=True, blank=True)
    Denisova = models.TextField(null=True, blank=True)
    Ensembl_geneid = models.TextField(null=True, blank=True)
    Ensembl_transcriptid = models.TextField(null=True, blank=True)
    Ensembl_proteinid = models.TextField(null=True, blank=True)
    aapos = models.TextField(null=True, blank=True)
    SIFT_score = models.TextField(null=True, blank=True)
    SIFT_converted_rankscore = models.TextField(null=True, blank=True)
    SIFT_pred = models.TextField(null=True, blank=True)
    Uniprot_acc_Polyphen2 = models.TextField(null=True, blank=True)
    Uniprot_id_Polyphen2 = models.TextField(null=True, blank=True)
    Uniprot_aapos_Polyphen2 = models.TextField(null=True, blank=True)
    Polyphen2_HDIV_score = models.TextField(null=True, blank=True)
    Polyphen2_HDIV_rankscore = models.TextField(null=True, blank=True)
    Polyphen2_HDIV_pred = models.TextField(null=True, blank=True)
    Polyphen2_HVAR_score = models.TextField(null=True, blank=True)
    Polyphen2_HVAR_rankscore = models.TextField(null=True, blank=True)
    Polyphen2_HVAR_pred = models.TextField(null=True, blank=True)
    LRT_score = models.TextField(null=True, blank=True)
    LRT_converted_rankscore = models.TextField(null=True, blank=True)
    LRT_pred = models.TextField(null=True, blank=True)
    LRT_Omega = models.TextField(null=True, blank=True)
    MutationTaster_score = models.TextField(null=True, blank=True)
    MutationTaster_converted_rankscore = models.TextField(null=True, blank=True)
    MutationTaster_pred = models.TextField(null=True, blank=True)
    MutationTaster_model = models.TextField(null=True, blank=True)
    MutationTaster_AAE = models.TextField(null=True, blank=True)
    MutationAssessor_UniprotID = models.TextField(null=True, blank=True)
    MutationAssessor_variant = models.TextField(null=True, blank=True)
    MutationAssessor_score = models.TextField(null=True, blank=True)
    MutationAssessor_score_rankscore = models.TextField(null=True, blank=True)
    MutationAssessor_pred = models.TextField(null=True, blank=True)
    FATHMM_score = models.TextField(null=True, blank=True)
    FATHMM_converted_rankscore = models.TextField(null=True, blank=True)
    FATHMM_pred = models.TextField(null=True, blank=True)
    PROVEAN_score = models.TextField(null=True, blank=True)
    PROVEAN_converted_rankscore = models.TextField(null=True, blank=True)
    PROVEAN_pred = models.TextField(null=True, blank=True)
    Transcript_id_VEST3 = models.TextField(null=True, blank=True)
    Transcript_var_VEST3 = models.TextField(null=True, blank=True)
    VEST3_score = models.TextField(null=True, blank=True)
    VEST3_rankscore = models.TextField(null=True, blank=True)
    MetaSVM_score = models.TextField(null=True, blank=True)
    MetaSVM_rankscore = models.TextField(null=True, blank=True)
    MetaSVM_pred = models.TextField(null=True, blank=True)
    MetaLR_score = models.TextField(null=True, blank=True)
    MetaLR_rankscore = models.TextField(null=True, blank=True)
    MetaLR_pred = models.TextField(null=True, blank=True)
    Reliability_index = models.TextField(null=True, blank=True)
    M_CAP_score = models.TextField(null=True, blank=True)
    M_CAP_rankscore = models.TextField(null=True, blank=True)
    M_CAP_pred = models.TextField(null=True, blank=True)
    REVEL_score = models.TextField(null=True, blank=True)
    REVEL_rankscore = models.TextField(null=True, blank=True)
    MutPred_score = models.TextField(null=True, blank=True)
    MutPred_rankscore = models.TextField(null=True, blank=True)
    MutPred_protID = models.TextField(null=True, blank=True)
    MutPred_AAchange = models.TextField(null=True, blank=True)
    MutPred_Top5features = models.TextField(null=True, blank=True)
    CADD_raw = models.TextField(null=True, blank=True)
    CADD_raw_rankscore = models.TextField(null=True, blank=True)
    CADD_phred = models.TextField(null=True, blank=True)
    DANN_score = models.TextField(null=True, blank=True)
    DANN_rankscore = models.TextField(null=True, blank=True)
    fathmm_MKL_coding_score = models.TextField(null=True, blank=True)
    fathmm_MKL_coding_rankscore = models.TextField(null=True, blank=True)
    fathmm_MKL_coding_pred = models.TextField(null=True, blank=True)
    fathmm_MKL_coding_group = models.TextField(null=True, blank=True)
    Eigen_coding_or_noncoding = models.TextField(null=True, blank=True)
    Eigen_raw = models.TextField(null=True, blank=True)
    Eigen_phred = models.TextField(null=True, blank=True)
    Eigen_PC_raw = models.TextField(null=True, blank=True)
    Eigen_PC_phred = models.TextField(null=True, blank=True)
    Eigen_PC_raw_rankscore = models.TextField(null=True, blank=True)
    GenoCanyon_score = models.TextField(null=True, blank=True)
    GenoCanyon_score_rankscore = models.TextField(null=True, blank=True)
    integrated_fitCons_score = models.TextField(null=True, blank=True)
    integrated_fitCons_score_rankscore = models.TextField(null=True, blank=True)
    integrated_confidence_value = models.TextField(null=True, blank=True)
    GM12878_fitCons_score = models.TextField(null=True, blank=True)
    GM12878_fitCons_score_rankscore = models.TextField(null=True, blank=True)
    GM12878_confidence_value = models.TextField(null=True, blank=True)
    H1_hESC_fitCons_score = models.TextField(null=True, blank=True)
    H1_hESC_fitCons_score_rankscore = models.TextField(null=True, blank=True)
    H1_hESC_confidence_value = models.TextField(null=True, blank=True)
    HUVEC_fitCons_score = models.TextField(null=True, blank=True)
    HUVEC_fitCons_score_rankscore = models.TextField(null=True, blank=True)
    HUVEC_confidence_value = models.TextField(null=True, blank=True)
    GERP_NR = models.TextField(null=True, blank=True)
    GERP_RS = models.TextField(null=True, blank=True)
    GERP_RS_rankscore = models.TextField(null=True, blank=True)
    phyloP100way_vertebrate = models.TextField(null=True, blank=True)
    phyloP100way_vertebrate_rankscore = models.TextField(null=True, blank=True)
    phyloP20way_mammalian = models.TextField(null=True, blank=True)
    phyloP20way_mammalian_rankscore = models.TextField(null=True, blank=True)
    phastCons100way_vertebrate = models.TextField(null=True, blank=True)
    phastCons100way_vertebrate_rankscore = models.TextField(null=True, blank=True)
    phastCons20way_mammalian = models.TextField(null=True, blank=True)
    phastCons20way_mammalian_rankscore = models.TextField(null=True, blank=True)
    SiPhy_29way_pi = models.TextField(null=True, blank=True)
    SiPhy_29way_logOdds = models.TextField(null=True, blank=True)
    SiPhy_29way_logOdds_rankscore = models.TextField(null=True, blank=True)
    Gp3_AC_1k = models.TextField(null=True, blank=True)
    Gp3_AF_1k = models.TextField(null=True, blank=True)
    Gp3_AFR_AC_1k = models.TextField(null=True, blank=True)
    Gp3_AFR_AF_1k = models.TextField(null=True, blank=True)
    Gp3_EUR_AC_1k = models.TextField(null=True, blank=True)
    Gp3_EUR_AF_1k = models.TextField(null=True, blank=True)
    Gp3_AMR_AC_1k = models.TextField(null=True, blank=True)
    Gp3_AMR_AF_1k = models.TextField(null=True, blank=True)
    Gp3_EAS_AC_1k = models.TextField(null=True, blank=True)
    Gp3_EAS_AF_1k = models.TextField(null=True, blank=True)
    Gp3_SAS_AC_1k = models.TextField(null=True, blank=True)
    Gp3_SAS_AF_1k = models.TextField(null=True, blank=True)
    TWINSUK_AC = models.TextField(null=True, blank=True)
    TWINSUK_AF = models.TextField(null=True, blank=True)
    ALSPAC_AC = models.TextField(null=True, blank=True)
    ALSPAC_AF = models.TextField(null=True, blank=True)
    ESP6500_AA_AC = models.TextField(null=True, blank=True)
    ESP6500_AA_AF = models.TextField(null=True, blank=True)
    ESP6500_EA_AC = models.TextField(null=True, blank=True)
    ESP6500_EA_AF = models.TextField(null=True, blank=True)
    ExAC_AC = models.TextField(null=True, blank=True)
    ExAC_AF = models.TextField(null=True, blank=True)
    ExAC_Adj_AC = models.TextField(null=True, blank=True)
    ExAC_Adj_AF = models.TextField(null=True, blank=True)
    ExAC_AFR_AC = models.TextField(null=True, blank=True)
    ExAC_AFR_AF = models.TextField(null=True, blank=True)
    ExAC_AMR_AC = models.TextField(null=True, blank=True)
    ExAC_AMR_AF = models.TextField(null=True, blank=True)
    ExAC_EAS_AC = models.TextField(null=True, blank=True)
    ExAC_EAS_AF = models.TextField(null=True, blank=True)
    ExAC_FIN_AC = models.TextField(null=True, blank=True)
    ExAC_FIN_AF = models.TextField(null=True, blank=True)
    ExAC_NFE_AC = models.TextField(null=True, blank=True)
    ExAC_NFE_AF = models.TextField(null=True, blank=True)
    ExAC_SAS_AC = models.TextField(null=True, blank=True)
    ExAC_SAS_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_Adj_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_Adj_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_AFR_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_AFR_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_AMR_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_AMR_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_EAS_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_EAS_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_FIN_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_FIN_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_NFE_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_NFE_AF = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_SAS_AC = models.TextField(null=True, blank=True)
    ExAC_nonTCGA_SAS_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_Adj_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_Adj_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_AFR_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_AFR_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_AMR_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_AMR_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_EAS_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_EAS_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_FIN_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_FIN_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_NFE_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_NFE_AF = models.TextField(null=True, blank=True)
    ExAC_nonpsych_SAS_AC = models.TextField(null=True, blank=True)
    ExAC_nonpsych_SAS_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_AFR_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_AFR_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_AFR_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_AMR_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_AMR_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_AMR_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_ASJ_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_ASJ_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_ASJ_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_EAS_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_EAS_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_EAS_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_FIN_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_FIN_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_FIN_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_NFE_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_NFE_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_NFE_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_SAS_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_SAS_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_SAS_AF = models.TextField(null=True, blank=True)
    gnomAD_exomes_OTH_AC = models.TextField(null=True, blank=True)
    gnomAD_exomes_OTH_AN = models.TextField(null=True, blank=True)
    gnomAD_exomes_OTH_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_AFR_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_AFR_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_AFR_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_AMR_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_AMR_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_AMR_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_ASJ_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_ASJ_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_ASJ_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_EAS_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_EAS_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_EAS_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_FIN_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_FIN_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_FIN_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_NFE_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_NFE_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_NFE_AF = models.TextField(null=True, blank=True)
    gnomAD_genomes_OTH_AC = models.TextField(null=True, blank=True)
    gnomAD_genomes_OTH_AN = models.TextField(null=True, blank=True)
    gnomAD_genomes_OTH_AF = models.TextField(null=True, blank=True)
    clinvar_rs = models.TextField(null=True, blank=True)
    clinvar_clnsig = models.TextField(null=True, blank=True)
    clinvar_trait = models.TextField(null=True, blank=True)
    clinvar_golden_stars = models.TextField(null=True, blank=True)
    Interpro_domain = models.TextField(null=True, blank=True)
    GTEx_V6p_gene = models.TextField(null=True, blank=True)
    GTEx_V6p_tissue = models.TextField(null=True, blank=True)


class Genome1kVariant(models.Model):
    pos_index = models.TextField(db_index=True)#ex. 1-326754756
    chrom = models.TextField(null=True, blank=True, db_index=True)
    pos = models.TextField(null=True, blank=True, db_index=True)
    rsid = models.TextField(null=True, blank=True)
    ref = models.TextField(null=True, blank=True, db_index=True)
    alt = models.TextField(null=True, blank=True, db_index=True)
    qual = models.TextField(null=True, blank=True)
    filter = models.TextField(null=True, blank=True)
    info = models.TextField(null=True, blank=True)
    format = models.TextField(null=True, blank=True)

class Genome1kVariantIndex(models.Model):
    index = models.TextField()#ex. 1-2387623-G-T REF ALT for each REF and ALT
    variant = models.ForeignKey(Genome1kVariant, on_delete=models.CASCADE)

class Genome1kSample(models.Model):
    name = models.TextField(null=True, blank=True)
    def __str__(self):
        return self.name

class Genome1kGenotype(models.Model):
    genotype = models.TextField(null=True, blank=True)

class Genome1kSampleVariant(models.Model):
    sample = models.ForeignKey(Genome1kSample, on_delete=models.CASCADE)
    variant = models.ForeignKey(Genome1kVariant, on_delete=models.CASCADE)
    genotype = models.ForeignKey(Genome1kGenotype, on_delete=models.CASCADE)
    # class Meta:
    #    managed = False
    #or add a fake primary key
    #https://groups.google.com/forum/#!topic/django-users/nRPURDAlgH0




from django.db import models

from django.contrib.auth.models import User

from datetime import datetime

from django.utils.text import slugify

from django.conf import settings

from django.template.defaultfilters import slugify

# Subclass AbstractUser
class UserGroup(models.Model):
    name = models.CharField(max_length=600)
    members = models.ManyToManyField(User, editable=True, related_name="members", blank=True)
    def __str__(self):
        return self.name


class Individual(models.Model):
    def get_upload_path(self, filename):
        if self.user != None:
            string = "%s/genomes/%s/%s/%s" % (settings.BASE_DIR, slugify(self.user.username), self.id, filename)#.replace(' ', '_')
        else:
            string = "%s/genomes/public/%s/%s" % (settings.BASE_DIR, self.id, filename)#.replace(' ', '_')
            print('string',string)
        return string
    user = models.ForeignKey(User, editable=False, null=True, on_delete=models.CASCADE)

    shared_with_users = models.ManyToManyField(User, editable=True, related_name="shared_with_users", blank=True)
    shared_with_groups = models.ManyToManyField(UserGroup, editable=True, related_name="shared_with_groups", blank=True)

    name = models.CharField(max_length=600)
    is_featured = models.BooleanField(default=True)
    is_public = models.BooleanField(default=False)
    vcf_file = models.FileField(upload_to=get_upload_path, blank=True, help_text="File Format: VCF",max_length=600)
    vcf_header = models.TextField(null=True, blank=True)
    status = models.CharField(max_length=100, blank=True, editable=False)
    n_variants = models.IntegerField(null=True, blank=True, editable=False)
    n_lines = models.IntegerField(null=True, blank=True, editable=False)

    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(null=True, blank=True)

    annotation_time = models.CharField(max_length=200, null=True, blank=True)
    insertion_time = models.CharField(max_length=200, null=True, blank=True)

    def __str__(self):
        return self.name

    # @models.permalink
    # def get_absolute_url(self):
    # 	return ('individual-new',)


    def save(self, *args, **kwargs):
        if not self.creation_date:
            self.creation_date = datetime.now()
        self.modified_date = datetime.now()
        return super(Individual, self).save(*args, **kwargs)

    # def delete(self, *args, **kwargs):
    #     super(Individual, self).delete(*args, **kwargs)

class Group(models.Model):
    name = models.CharField(max_length=128)
    members = models.ManyToManyField(Individual,
        related_name='group_members')
    def __str__(self):
        return self.name

#from individuals.tasks import PopulateControls

class ControlGroup(models.Model):
    def get_upload_path(self, filename):
        string = "upload/controls/%s/%s" % (self.id, filename)#.replace(' ', '_')
        return string
    name = models.CharField(max_length=600)
    vcf_file = models.FileField(upload_to=get_upload_path, blank=True, help_text="File Format: VCF",max_length=600)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        #populate
        #return
        super(ControlGroup, self).save(*args, **kwargs)
        PopulateControls.delay(self.id)


class ControlVariant(models.Model):

    controlgroup = models.ForeignKey(ControlGroup, on_delete=models.CASCADE)
    index = models.TextField(db_index=True)#ex. 1-2387623-G-T



# -*- coding: utf-8 -*-
"""Settings for Django-Select2."""
from __future__ import absolute_import, unicode_literals

from appconf import AppConf
from django.conf import settings  # NOQA

__all__ = ('settings', 'Select2Conf')


class Select2Conf(AppConf):
    """Settings for Django-Select2."""

    CACHE_BACKEND = 'default'
    """
    Django-Select2 uses Django's cache to sure a consistent state across multiple machines.

    Example of settings.py::

        CACHES = {
            "default": {
                "BACKEND": "django_redis.cache.RedisCache",
                "LOCATION": "redis://127.0.0.1:6379/1",
                "OPTIONS": {
                    "CLIENT_CLASS": "django_redis.client.DefaultClient",
                }
            },
            'select2': {
                'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
                'LOCATION': '127.0.0.1:11211',
            }
        }

        # Set the cache backend to select2
        SELECT2_CACHE_BACKEND = 'select2'

    .. tip:: To ensure a consistent state across all you machines you need to user
        a consistent external cache backend like Memcached, Redis or a database.

    .. note:: The timeout of select2's caching backend determines
        how long a browser session can last.
        Once widget is dropped from the cache the json response view will return a 404.
    """
    CACHE_PREFIX = 'select2_'
    """
    If you caching backend doesn't support multiple databases
    you can isolate select2 using the cache prefix setting.
    It has set `select2_` as a default value, which you can change if needed.
    """

    JS = '//cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.min.js'
    """
    The URI for the Select2 JS file. By default this points to the Cloudflare CDN.

    If you want to select the version of the JS library used, or want to serve it from
    the local 'static' resources, add a line to your settings.py like so::

        SELECT2_JS = 'assets/js/select2.min.js'

    .. tip:: Change this setting to a local asset in your development environment to
        develop without an Internet connection.
    """

    CSS = '//cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/css/select2.min.css'
    """
    The URI for the Select2 CSS file. By default this points to the Cloudflare CDN.

    If you want to select the version of the library used, or want to serve it from
    the local 'static' resources, add a line to your settings.py like so::

        SELECT2_CSS = 'assets/css/select2.css'

    .. tip:: Change this setting to a local asset in your development environment to
        develop without an Internet connection.
    """

    class Meta:
        """Prefix for all Django-Select2 settings."""

        prefix = 'SELECT2'



# -*- coding: utf-8 -*-

# Create your tasks here
from __future__ import absolute_import, unicode_literals
from celery import shared_task

# from celery.task import Task
# from celery.registry import tasks
from django.db import transaction
from django.core.files import File
# from celery import task

from individuals.models import *

from variants.models import *

# from mysql_bulk_insert import bulk_insert
from django.shortcuts import render, get_object_or_404
import os
import datetime
from django.core.mail import send_mail
# from snpedia.models import *
from diseases.models import HGMDMutation
from django.conf import settings

import zipfile
import gzip
import pickle
import tarfile
from collections import OrderedDict


import json
import vcf

from datetime import timedelta
from django.template.defaultfilters import slugify


@shared_task()
def clean_individuals():
    print("Running periodic task!")
    individuals = Individual.objects.filter(user=None)
    for individual in individuals:
        time_difference = datetime.datetime.now()-individual.creation_date
        if time_difference.days > 0:
            #delete individuals
            os.system('rm -rf %s/genomes/public/%s' % (settings.BASE_DIR, individual_id))
            individual.delete()

@shared_task()
def VerifyVCF(individual_id):
    print('Verify VCF...')

    individual = get_object_or_404(Individual, pk=individual_id)
    print(individual.vcf_file)
    filename = str(individual.vcf_file.name.split('/')[-1])

    if individual.user:
        path  = '%s/genomes/%s/%s' % (settings.BASE_DIR, slugify(individual.user.username), individual.id)
    else:
        path  = '%s/genomes/public/%s' % (settings.BASE_DIR, individual.id)

    new_path = '/'.join(path.split('/')[:-1])
    
    print(new_path)

    os.chdir(path)
    
    print('filename', filename)

    if filename.endswith('.vcf'):
        command = 'cp %s sample.vcf' % (filename)
        os.system(command)
    elif filename.endswith('.gz'):
        command = 'gunzip -c -d %s > sample.vcf' % (filename)
        os.system(command)
    elif filename.endswith('.zip'):
        command = 'unzip -p %s > sample.vcf' % (filename)
        os.system(command)
    elif filename.endswith('.rar'):
        command = 'unrar e %s' % (filename)
        os.system(command)
        #now change filename to sample.vcf
        command = 'mv %s sample.vcf' % (filename.replace('.rar', ''))
        os.system(command)


    vcf_reader = vcf.Reader(open('sample.vcf', 'r'))
    n_samples = len(vcf_reader.samples)
    print('n_samples', n_samples)

    if n_samples > 1:
        #extract individuals and create new users
        for sample in vcf_reader.samples:
            print(sample)
            command = "bcftools view -c 1 -s %s sample.vcf  > %s.vcf" % (sample, sample)
            print(command)
            os.system(command)
        #now rename original sample
        first_sample = vcf_reader.samples[0]
        original_name = individual.name
        individual.name += ' %s' % (first_sample)
        individual.vcf_file = "%s/%s/%s.vcf" % (new_path, individual.id, first_sample)
        # individual.save()
        AnnotateVariants.delay(individual.id)
        #create other samples
        for sample in vcf_reader.samples[1:]:
            print(sample)
            new_individual = Individual.objects.create(user=individual.user, status='new')
            new_individual.name = original_name + ' %s' % (sample)
            # new_individual.save()
            output_folder = '../%s' % (new_individual.id)
            if not os.path.exists(output_folder):
                os.makedirs(output_folder)
                os.chmod(output_folder, 0o777)
            command = 'mv %s.vcf %s' % (sample, output_folder)
            os.system(command)
            new_individual.vcf_file = '%s/%s/%s.vcf' % (new_path, new_individual.id, sample)
            # new_individual.save()
            # AnnotateVariants.delay(new_individual.id)
    else:
        AnnotateVariants.delay(individual_id)
    #check if VCF is multisample
    #if so extract individuals and create other individual models
    #if not send it to be annotated

@shared_task()
def AnnotateVariants(individual_id):

    print('Annotation Started!!!')
    start = datetime.datetime.now()

    individual = get_object_or_404(Individual, pk=individual_id)

    individual.status = 'running'
    individual.save()
    

    #chdir into folder
    # print 'individual.vcf_file.name'
    # print individual.vcf_file.name
    if individual.user:
        path  = '%s/genomes/%s/%s' % (settings.BASE_DIR,  slugify(individual.user.username), individual.id)
        email = individual.user.email
    else:
        path  = '%s/genomes/public/%s' % (settings.BASE_DIR, individual.id)
        email = 'raonyguimaraes@gmail.com'
    orig_path = os.getcwd()
    #change to path for the individual folder
    os.chdir(path)
    # print(os.getcwd())

    #delete annotation folder before start annotating
    command = 'rm -rf ann_*'
    os.system(command)

    filename = str(individual.vcf_file.name.split('/')[-1])
    print(filename)
    #deal with different types of compressed files
    #ex. zip, vcf, gz, rar
    #check if user uploaded a compressed vcf
    if filename.endswith('.vcf'):
        command = 'cp %s sample.vcf' % (filename)
        os.system(command)
    elif filename.endswith('.tar.gz'):
        print('targz')
        tar = tarfile.open(filename, "r:gz")
        for tarinfo in tar:
            #print(tarinfo.name, "is", tarinfo.size, "bytes in size and is", end="")
            if tarinfo.name.endswith('.vcf'):
                if not os.path.exists('outdir'):
                    os.mkdir('outdir')
                tar.extract(tarinfo.name, 'outdir')
                vcfs = os.listdir('outdir')
                command = 'cp outdir/%s sample.vcf' % (vcfs[0])
                print(command)
                os.system(command)
    
    if filename.endswith('.vcf.gz'):
        command = 'gunzip -c -d %s > sample.vcf' % (filename)
        os.system(command)
    if filename.endswith('.zip'):
        command = 'unzip -p %s > sample.vcf' % (filename)
        os.system(command)
    if filename.endswith('.rar'):
        command = 'unrar e %s' % (filename)
        os.system(command)
        #now change filename to sample.vcf
        command = 'mv %s sample.vcf' % (filename.replace('.rar', ''))
        os.system(command)


    #     individual.vcf_file.name = individual.vcf_file.name.replace('.zip', '.vcf')


    # print(os.getcwd())
    if os.path.exists('sample.vcf'):
        command = 'pynnotator -i sample.vcf'
        os.system(command)

    #get sample name using pyvcf

    vcf_filename = os.path.splitext(os.path.basename(str(filename)))[0]

    # create a folder for the annotation if it doesn't exists,
    # or delete and create if the folder already exists

    #first check if annotation succedded
    annotation_final_file = 'ann_sample/annotation.final.vcf'

    # print('checking destination ', annotation_final_file)
    stop = datetime.datetime.now()
    elapsed = stop - start

    individual.annotation_time = elapsed

    if os.path.exists(annotation_final_file):

        individual.status = 'annotated'
        #send email
        message = """The file %s was annotated with success!\n
It took %s to execute. \n
Now we need to insert this data to the database.
                """ % (individual.name, elapsed)
        send_mail('[Mendel,MD] Annotation Completed!', message, 'mendelmd1@gmail.com',
                  ['raonyguimaraes@gmail.com', email], fail_silently=False)
        #delete ann folder
        command = 'rm -rf ann_*'
        # os.system(command)
        #delete sample
        command = 'rm -rf sample.vcf'
        os.system(command)

        #zip, and delete annotation folder

        command = 'zip annotation.final.vcf.zip ann_sample/annotation.final.vcf'
        os.system(command)

        PopulateVariants.delay(individual.id)

        if individual.vcf_file.name.endswith(".vcf"):
            command = 'bgzip %s' % (filename)
            os.system(command)
            individual.vcf_file.name = '%s.gz' % (individual.vcf_file.name)

    else:
        individual.status = 'failed'
        message = """The Individual %s failed to be annotated!\n
                It took %s to execute.
                """ % (individual.name, elapsed)
        send_mail('[Mendel,MD] Annotation Failed!', message, 'mendelmd1@gmail.com',
                  ['raonyguimaraes@gmail.com'], fail_silently=False)

    individual.save()
    os.chdir(settings.BASE_DIR)

def treat_float_max(float_string):
    max_value = -100
    values = float_string.split(',')
    for value in values:
        if value != '.':
            float_value = float(value)
            if float_value > max_value:
                max_value = float_value
    return max_value

def treat_float_min(float_string):
    minimum_value = 1
    values = float_string.split(',')
    for value in values:
        if value != '.':
            float_value = float(value)
            if float_value < minimum_value:
                minimum_value = float_value
    return minimum_value

def parse_vcf(line):

    """
    This function is responsible for parsing the VCF file that is generated by our annotator pipeline and
    return a dictionary with the fields to be included in the database

    Receives variant_line and return variant dict
    """

    variant = OrderedDict()
    #parse first VCF Lines
    variant_line = line.strip().split('\t')
    # print(variant_line)


    variant['chr'] = variant_line[0]

    #treat vcfs with chr on the begining of chromossome names
    if variant['chr'].startswith('chr'):
      variant['chr'] = variant['chr'].replace('chr', '')

    variant['pos'] = variant_line[1]
    variant['variant_id'] = variant_line[2]
    variant['ref'] = variant_line[3]
    variant['alt'] = variant_line[4]

    #this is form strange vcfs without qual values
    if variant_line[5] == "-1" or variant_line[5] == ".":
        variant['qual'] = 0.0
    else:
        variant['qual'] = variant_line[5]

    variant['filter'] = variant_line[6]

    variant['format'] = variant_line[-2].split(':')

    variant['genotype_col'] = variant_line[-1].split(':')

    variant['genotype'] = variant['genotype_col'][0]

    # print 'genotype', variant['genotype_col'], variant['genotype']

    if variant['genotype'] != './.':
        #fix because of isaac variant caller, there is no DP
        if 'DP' in variant['format']:
            value = variant['genotype_col'][variant['format'].index('DP')]
            if value.isdigit():
                variant['read_depth'] = int(value)
            else:
                # print('not an integer', value, variant['pos'])
                variant['read_depth'] = 0
        else:
            variant['read_depth'] = 0
    else:
        variant['read_depth'] = 0

    #annotations
    info = variant_line[7]
    string = info.split(';')

    #create a dict with all annotations at INFO field
    information = {}
    # dbnfsp_fields = []
    for element in string:
      #get all tags from line
      element = element.split('=')
      tag = element[0]
      if len(element) > 1:
        information[tag] = element[1]#.decode("utf-8", "ignore")
      else:
        information[tag] = tag
        # if element[0].startswith('dbNSFP'):
        #     dbnfsp_fields.append(element)
    if 'HET' in  information:
        variant['mutation_type'] = 'HET'
    elif 'HOM' in  information:
        variant['mutation_type'] = 'HOM'
    else:
        variant['mutation_type'] = None

    if 'VARTYPE' in  information:
        variant['vartype'] = information['VARTYPE']
    else:
        variant['vartype'] = None

    #serialize object with pickle
    variant['info'] = json.dumps(information)#.decode("utf-8", "ignore")

    # print 'information dict'
    # print information

    #parse VEP
    #Allele|Consequence|IMPACT|SYMBOL|Gene|Feature_type|Feature|BIOTYPE|EXON|INTRON|HGVSc|HGVSp|cDNA_position|CDS_position|Protein_position|Amino_acids|Codons|Existing_variation|DISTANCE|STRAND|FLAGS|SYMBOL_SOURCE|HGNC_ID|SIFT|PolyPhen

    if 'CSQ' in information:
        vep = OrderedDict()
        vep_list = information['CSQ'].split('|')

        vep['Allele'] = vep_list[0]
        vep['Consequence'] = vep_list[1]
        vep['IMPACT'] = vep_list[2]
        vep['SYMBOL'] = vep_list[3]
        vep['Gene'] = vep_list[4]
        vep['Feature_type'] = vep_list[5]
        vep['Feature'] = vep_list[6]
        vep['BIOTYPE'] = vep_list[7]
        vep['EXON'] = vep_list[8]
        vep['INTRON'] = vep_list[9]
        vep['HGVSc'] = vep_list[10]
        vep['HGVSp'] = vep_list[11]

        vep['cDNA_position'] = vep_list[12]
        vep['CDS_position'] = vep_list[13]
        vep['Protein_position'] = vep_list[14]
        vep['Amino_acids'] = vep_list[15]
        vep['Codons'] = vep_list[16]
        vep['Existing_variation'] = vep_list[17]
        vep['DISTANCE'] = vep_list[18]
        vep['STRAND'] = vep_list[19]
        vep['FLAGS'] = vep_list[20]

        vep['SYMBOL_SOURCE'] = vep_list[21]
        vep['HGNC_ID'] = vep_list[22]
        vep['sift'] = vep_list[23]
        vep['polyphen2'] = vep_list[24]


        # print vep
        variant['vep'] = vep
        variant['gene'] = vep['SYMBOL']

    else:
        variant['gene'] = None

    #treat vep sift, polyphen
    csq_list = ['sift', 'polyphen2']
    for tag in csq_list:
        tag_pred = '%s_pred' % (tag)

        variant[tag]=None
        variant[tag_pred]=None

        if 'vep' in variant:
            if vep[tag] != '':
                # print('veptag', tag, vep[tag])
                value = vep[tag].split('(')
                variant[tag] = float(value[1].replace(')',''))
                variant[tag_pred] = value[0]

    #parse SNPEFF
    ##INFO=<ID=EFF,Number=.,Type=String,Description="Predicted effects for this variant.Format: 'Effect ( Effect_Impact | Functional_Class | Codon_Change | Amino_Acid_Change| Amino_Acid_length | Gene_Name | Transcript_BioType | Gene_Coding | Transcript_ID | Exon_Rank  | Genotype_Number [ | ERRORS | WARNINGS ] )' ">

    if 'EFF' in information:
        # print information['EFF']
        variant['snpeff'] = []
        effects = information['EFF'].split(',')
        # print len(effects), effects
        for ann in effects:
            snpeff = OrderedDict()

            eff_str = ann

            eff_str_list = eff_str.split('(')
            effects = eff_str_list[1].split('|')
            #EFF,Number=.,Type=String,Description="Predicted effects for this variant.Format: 'Effect ( Effect_Impact | Functional_Class | Codon_Change | Amino_Acid_Change| Amino_Acid_length | Gene_Name | Transcript_BioType | Gene_Coding | Transcript_ID | Exon_Rank  | Genotype [ | ERRORS | WARNINGS ] )' ">
            snpeff['effect'] = eff_str_list[0]
            snpeff['impact'] = effects[0]
            snpeff['func_class'] = effects[1]
            snpeff['codon_change'] = effects[2]
            snpeff['aa_change'] = effects[3]
            snpeff['aa_len'] = effects[4]
            snpeff['gene_name'] = effects[5]
            snpeff['biotype'] = effects[6]
            snpeff['gene_coding'] = effects[7]
            snpeff['transcript_id'] = effects[8]
            snpeff['exon_rank'] = effects[9]
            snpeff['genotype_number'] = effects[10].split(')')[0]
            variant['snpeff'].append(snpeff)

    float_list = ['genomes1k.AF']
    for tag in float_list:
        if tag in information:
            variant[tag] = treat_float_min(information[tag])
        else:
             variant[tag] = None

    #Special TAG FROM ESP6500
    try:
      variant['esp6500.MAF'] = float(information['esp6500.MAF'].split(',')[-1]) / 100.0
    except (KeyError):
      variant['esp6500.MAF'] = None
    #dbsnp CAF
    try:
        caf = information['dbsnp.CAF']
        caf = caf.replace('[', '').replace(']', '').split(',')
        # print 'caf', caf
        floats = []
        for x in caf:
            if x != '.':
                floats.append(float(x))
        # print sorted(floats, key=float, reverse=True)
        #get always second element from this list
        variant['dbsnp.MAF'] = floats[1]
    except (KeyError):
        variant['dbsnp.MAF'] = None

    if 'dbsnp.dbSNPBuildID' in information:
        information['dbsnp.dbSNPBuildID'] = min(information['dbsnp.dbSNPBuildID'].split(','))
        variant['dbsnp_build'] = int(information['dbsnp.dbSNPBuildID'])
    else:
        variant['dbsnp_build'] = None

    dbnfsp_fields = ['dbNSFP_SIFT_score', 'dbNSFP_SIFT_converted_rankscore', 'dbNSFP_SIFT_pred', 'dbNSFP_Uniprot_acc_Polyphen2', 'dbNSFP_Uniprot_id_Polyphen2', 'dbNSFP_Uniprot_aapos_Polyphen2', 'dbNSFP_Polyphen2_HDIV_score', 'dbNSFP_Polyphen2_HDIV_rankscore', 'dbNSFP_Polyphen2_HDIV_pred', 'dbNSFP_Polyphen2_HVAR_score', 'dbNSFP_Polyphen2_HVAR_rankscore', 'dbNSFP_Polyphen2_HVAR_pred', 'dbNSFP_LRT_score', 'dbNSFP_LRT_converted_rankscore', 'dbNSFP_LRT_pred', 'dbNSFP_LRT_Omega', 'dbNSFP_MutationTaster_score', 'dbNSFP_MutationTaster_converted_rankscore', 'dbNSFP_MutationTaster_pred', 'dbNSFP_MutationTaster_model', 'dbNSFP_MutationTaster_AAE', 'dbNSFP_MutationAssessor_UniprotID', 'dbNSFP_MutationAssessor_variant', 'dbNSFP_MutationAssessor_score', 'dbNSFP_MutationAssessor_rankscore', 'dbNSFP_MutationAssessor_pred', 'dbNSFP_FATHMM_score', 'dbNSFP_FATHMM_converted_rankscore', 'dbNSFP_FATHMM_pred', 'dbNSFP_PROVEAN_score', 'dbNSFP_PROVEAN_converted_rankscore', 'dbNSFP_PROVEAN_pred', 'dbNSFP_Transcript_id_VEST3', 'dbNSFP_Transcript_var_VEST3', 'dbNSFP_VEST3_score', 'dbNSFP_VEST3_rankscore', 'dbNSFP_MetaSVM_score', 'dbNSFP_MetaSVM_rankscore', 'dbNSFP_MetaSVM_pred', 'dbNSFP_MetaLR_score', 'dbNSFP_MetaLR_rankscore', 'dbNSFP_MetaLR_pred', 'dbNSFP_Reliability_index', 'dbNSFP_M-CAP_score', 'dbNSFP_M-CAP_rankscore', 'dbNSFP_M-CAP_pred', 'dbNSFP_REVEL_score', 'dbNSFP_REVEL_rankscore', 'dbNSFP_MutPred_score', 'dbNSFP_MutPred_rankscore', 'dbNSFP_MutPred_protID', 'dbNSFP_MutPred_AAchange', 'dbNSFP_MutPred_Top5features', 'dbNSFP_CADD_raw', 'dbNSFP_CADD_raw_rankscore', 'dbNSFP_CADD_phred', 'dbNSFP_DANN_score', 'dbNSFP_DANN_rankscore', 'dbNSFP_fathmm-MKL_coding_score', 'dbNSFP_fathmm-MKL_coding_rankscore', 'dbNSFP_fathmm-MKL_coding_pred', 'dbNSFP_fathmm-MKL_coding_group', 'dbNSFP_Eigen_coding_or_noncoding', 'dbNSFP_Eigen-raw', 'dbNSFP_Eigen-phred', 'dbNSFP_Eigen-PC-raw', 'dbNSFP_Eigen-PC-phred', 'dbNSFP_Eigen-PC-raw_rankscore', 'dbNSFP_GenoCanyon_score', 'dbNSFP_GenoCanyon_score_rankscore', 'dbNSFP_integrated_fitCons_score', 'dbNSFP_integrated_fitCons_rankscore', 'dbNSFP_integrated_confidence_value', 'dbNSFP_GM12878_fitCons_score', 'dbNSFP_GM12878_fitCons_rankscore', 'dbNSFP_GM12878_confidence_value', 'dbNSFP_H1-hESC_fitCons_score', 'dbNSFP_H1-hESC_fitCons_rankscore', 'dbNSFP_H1-hESC_confidence_value', 'dbNSFP_HUVEC_fitCons_score', 'dbNSFP_HUVEC_fitCons_rankscore', 'dbNSFP_clinvar_rs', 'dbNSFP_clinvar_clnsig', 'dbNSFP_clinvar_trait', 'dbNSFP_clinvar_golden_stars']
    dbnfsp = {}

    for field in dbnfsp_fields:
        if field in information:
            dbnfsp[field] = information[field]
        else:
            dbnfsp[field] = None
    # print(dbnfsp)

    variant['dbNSFP'] = dbnfsp

    # #cadd
    if 'dbNSFP_CADD_raw' in information:
        variant['cadd'] = treat_float_max(dbnfsp['dbNSFP_CADD_raw'])
    else:
        variant['cadd']=None

    if 'dbNSFP_M-CAP_score' in information:
        variant['mcap'] = treat_float_max(dbnfsp['dbNSFP_M-CAP_score'])
    else:
        variant['mcap']=None




    #is at OMIM
    if 'clinvar.OM' in information:
        variant['is_at_omim'] = True
    else:
        variant['is_at_omim'] = False
    #is at HGMD
    if 'HGMD' in information:
        variant['is_at_hgmd'] = True
        variant['hgmd_entries'] = information['HGMD']
    else:
        variant['is_at_hgmd'] = False
        variant['hgmd_entries'] = None

    if 'HI_PREDICTIONS' in information:
        variant['hi_index_str'] = information['HI_PREDICTIONS']
    else:
        variant['hi_index_str'] = None

    #create unique index for each chr-pos-genotype
    #genotype, ref and alt
    genotype_list = []
    genotype_list.append(variant['ref'])
    for item in variant['alt'].split(','):
        genotype_list.append(item)

    #create individual genotype list
    ind_genotype_list = []
    if len(variant['genotype']) > 1:
        if variant['genotype'][0] != '.':
            ind_genotype_list.append(genotype_list[int(variant['genotype'][0])])
        else:
            ind_genotype_list.append(genotype_list[0])
        if variant['genotype'][-1] != '.':
            ind_genotype_list.append(genotype_list[int(variant['genotype'][-1])])
        else:
            ind_genotype_list.append(genotype_list[1])

    else:
        ind_genotype_list.append(genotype_list[0])
    ind_genotype_list = sorted(ind_genotype_list)

    variant['index'] = '%s-%s-%s' % (variant['chr'], variant['pos'], "-".join(ind_genotype_list))
    variant['pos_index'] = '%s-%s' % (variant['chr'], variant['pos'])

    # print index
    # print(variant)
    return variant

@shared_task()
def PopulateVariants(individual_id):
    # print os.getcwd()

    #delete variants from individual before inserting
    individual = get_object_or_404(Individual, pk=individual_id)
    Variant.objects.filter(individual=individual).delete()
    #SnpeffAnnotation.objects.filter(individual=individual).delete()
    #VEPAnnotation.objects.filter(individual=individual).delete()

    filepath = os.path.dirname(str(individual.vcf_file.name))
    filename = os.path.basename(str(individual.vcf_file.name))

    print('Populating %s %s' % (individual.id, filename))
    os.system('echo "Populating Individual %s"' % (individual.name))
    # print filepath
    # print filename
    #print basename
    #gzip. and .gz
    # data = open('%s/%s/ann_sample/annotation.final.vcf' % (path, filepath), 'r')

    z = zipfile.ZipFile('%s/annotation.final.vcf.zip' % (filepath), 'r')
    data = z.open('ann_sample/annotation.final.vcf', 'r')

    #print 'Populating from file %s.fullannotation.vcf' % (basename)


    start = datetime.datetime.now()

    count = 0

    variants = []
    count2 = 0

    snpeff_dict = {}
    vep_dict = {}

    for line in data:
        # print(line)
        line = line.decode("utf-8", "ignore")
        # print(line)
        # print('Hello')
        if line != '':
            if not line.startswith('#'):

                count += 1
                count2 += 1

                #bulk insert variants objects
                if count == 10000:
                    # print "Inserting %s " % (count2),
                    Variant.objects.bulk_create(variants)
                    # print ' Done!'
                    count = 0
                    variants = []

                #now parse
                variant = parse_vcf(line)
                #print(variant)
                #variant_dict['individual_id'] = individual.id
                # print 'index', variant
                # variant.snpeff.add(snpeff)
                variant_obj = Variant(
                individual=individual,
                index=variant['index'],
                pos_index=variant['pos_index'],
                chr=variant['chr'],
                pos=variant['pos'],
                variant_id=variant['variant_id'],
                ref=variant['ref'],
                alt=variant['alt'],
                qual=variant['qual'],
                filter=variant['filter'],
                info=variant['info'],
                genotype=variant['genotype'],
                genotype_col=variant['genotype_col'],
                format=variant['format'],
                read_depth=variant['read_depth'],
                gene=variant['gene'],
                mutation_type=variant['mutation_type'],
                vartype=variant['vartype'],
                genomes1k_maf=variant['genomes1k.AF'],
                dbsnp_maf=variant['dbsnp.MAF'],
                esp_maf=variant['esp6500.MAF'],
                dbsnp_build=variant['dbsnp_build'],
                sift=variant['sift'],
                sift_pred=variant['sift_pred'],
                polyphen2=variant['polyphen2'],
                polyphen2_pred=variant['polyphen2_pred'],
                # condel=variant['condel'],
                # condel_pred=variant['condel_pred'],
                cadd=variant['cadd'],
                # dann=variant['dann'],
                is_at_omim=variant['is_at_omim'],
                is_at_hgmd=variant['is_at_hgmd'],
                hgmd_entries=variant['hgmd_entries'],
                hi_index_str=variant['hi_index_str'],
                )

                # print variant['index']
                # variant_obj.save()
                #parse snpeff

                if 'snpeff' in variant:
                    #for snpeff in variant['snpeff']:
                        #snpeff = SnpeffAnnotation(
                    variant_obj.snpeff_effect=variant['snpeff'][0]['effect']
                    variant_obj.snpeff_impact=variant['snpeff'][0]['impact']
                    variant_obj.snpeff_func_class=variant['snpeff'][0]['func_class']
                    variant_obj.snpeff_codon_change=variant['snpeff'][0]['codon_change']
                    variant_obj.snpeff_aa_change=variant['snpeff'][0]['aa_change']
                    # variant_obj.snpeff_aa_len=variant['snpeff'][0]['aa_len']
                    variant_obj.snpeff_gene_name=variant['snpeff'][0]['gene_name']
                    variant_obj.snpeff_biotype=variant['snpeff'][0]['biotype']
                    variant_obj.snpeff_gene_coding=variant['snpeff'][0]['gene_coding']
                    variant_obj.snpeff_transcript_id=variant['snpeff'][0]['transcript_id']
                    variant_obj.snpeff_exon_rank=variant['snpeff'][0]['exon_rank']
                    # variant_obj.snpeff_genotype_number=variant['snpeff'][0]['genotype_number']
                    #)
                    #snpeff_dict[variant['index']] = snpeff

                #parse vep
                if 'vep' in variant:
                    #vep = VEPAnnotation(
                    variant_obj.vep_allele=variant['vep']['Allele']
                    variant_obj.vep_gene=variant['vep']['Gene']
                    variant_obj.vep_feature=variant['vep']['Feature']
                    variant_obj.vep_feature_type=variant['vep']['Feature_type']
                    variant_obj.vep_consequence=variant['vep']['Consequence']
                    variant_obj.vep_cdna_position=variant['vep']['cDNA_position']
                    variant_obj.vep_cds_position=variant['vep']['CDS_position']
                    variant_obj.vep_protein_position=variant['vep']['Protein_position']
                    variant_obj.vep_amino_acids=variant['vep']['Amino_acids']
                    variant_obj.vep_codons=variant['vep']['Codons']
                    variant_obj.vep_existing_variation=variant['vep']['Existing_variation']
                    variant_obj.vep_distance=variant['vep']['DISTANCE']
                    variant_obj.vep_strand=variant['vep']['STRAND']
                    variant_obj.vep_symbol=variant['vep']['SYMBOL']
                    variant_obj.vep_symbol_source=variant['vep']['SYMBOL_SOURCE']
                    variant_obj.vep_sift=variant['vep']['sift']
                    variant_obj.vep_polyphen=variant['vep']['polyphen2']
                    # variant_obj.vep_condel=variant['vep']['condel']
                    # variant_obj.rf_score=variant['vep']['rf_score']
                    # variant_obj.ada_score=variant['vep']['ada_score']

                    #)
                    #vep_dict[variant['index']] = vep

                if 'dbNSFP' in variant:

                    variant_obj.SIFT_score = variant['dbNSFP']['dbNSFP_SIFT_score']
                    variant_obj.SIFT_converted_rankscore = variant['dbNSFP']['dbNSFP_SIFT_converted_rankscore']
                    variant_obj.SIFT_pred = variant['dbNSFP']['dbNSFP_SIFT_pred']
                    variant_obj.Uniprot_acc_Polyphen2 = variant['dbNSFP']['dbNSFP_Uniprot_acc_Polyphen2']
                    variant_obj.Uniprot_id_Polyphen2 = variant['dbNSFP']['dbNSFP_Uniprot_id_Polyphen2']
                    variant_obj.Uniprot_aapos_Polyphen2 = variant['dbNSFP']['dbNSFP_Uniprot_aapos_Polyphen2']
                    variant_obj.Polyphen2_HDIV_score = variant['dbNSFP']['dbNSFP_Polyphen2_HDIV_score']
                    variant_obj.Polyphen2_HDIV_rankscore = variant['dbNSFP']['dbNSFP_Polyphen2_HDIV_rankscore']
                    variant_obj.Polyphen2_HDIV_pred = variant['dbNSFP']['dbNSFP_Polyphen2_HDIV_pred']
                    variant_obj.Polyphen2_HVAR_score = variant['dbNSFP']['dbNSFP_Polyphen2_HVAR_score']
                    variant_obj.Polyphen2_HVAR_rankscore = variant['dbNSFP']['dbNSFP_Polyphen2_HVAR_rankscore']
                    variant_obj.Polyphen2_HVAR_pred = variant['dbNSFP']['dbNSFP_Polyphen2_HVAR_pred']
                    variant_obj.LRT_score = variant['dbNSFP']['dbNSFP_LRT_score']
                    variant_obj.LRT_converted_rankscore = variant['dbNSFP']['dbNSFP_LRT_converted_rankscore']
                    variant_obj.LRT_pred = variant['dbNSFP']['dbNSFP_LRT_pred']
                    variant_obj.LRT_Omega = variant['dbNSFP']['dbNSFP_LRT_Omega']
                    variant_obj.MutationTaster_score = variant['dbNSFP']['dbNSFP_MutationTaster_score']
                    variant_obj.MutationTaster_converted_rankscore = variant['dbNSFP']['dbNSFP_MutationTaster_converted_rankscore']
                    variant_obj.MutationTaster_pred = variant['dbNSFP']['dbNSFP_MutationTaster_pred']
                    variant_obj.MutationTaster_model = variant['dbNSFP']['dbNSFP_MutationTaster_model']
                    variant_obj.MutationTaster_AAE = variant['dbNSFP']['dbNSFP_MutationTaster_AAE']
                    variant_obj.MutationAssessor_UniprotID = variant['dbNSFP']['dbNSFP_MutationAssessor_UniprotID']
                    variant_obj.MutationAssessor_variant = variant['dbNSFP']['dbNSFP_MutationAssessor_variant']
                    variant_obj.MutationAssessor_score = variant['dbNSFP']['dbNSFP_MutationAssessor_score']
                    variant_obj.MutationAssessor_rankscore = variant['dbNSFP']['dbNSFP_MutationAssessor_rankscore']
                    variant_obj.MutationAssessor_pred = variant['dbNSFP']['dbNSFP_MutationAssessor_pred']
                    variant_obj.FATHMM_score = variant['dbNSFP']['dbNSFP_FATHMM_score']
                    variant_obj.FATHMM_converted_rankscore = variant['dbNSFP']['dbNSFP_FATHMM_converted_rankscore']
                    variant_obj.FATHMM_pred = variant['dbNSFP']['dbNSFP_FATHMM_pred']
                    variant_obj.PROVEAN_score = variant['dbNSFP']['dbNSFP_PROVEAN_score']
                    variant_obj.PROVEAN_converted_rankscore = variant['dbNSFP']['dbNSFP_PROVEAN_converted_rankscore']
                    variant_obj.PROVEAN_pred = variant['dbNSFP']['dbNSFP_PROVEAN_pred']
                    variant_obj.Transcript_id_VEST3 = variant['dbNSFP']['dbNSFP_Transcript_id_VEST3']
                    variant_obj.Transcript_var_VEST3 = variant['dbNSFP']['dbNSFP_Transcript_var_VEST3']
                    variant_obj.VEST3_score = variant['dbNSFP']['dbNSFP_VEST3_score']
                    variant_obj.VEST3_rankscore = variant['dbNSFP']['dbNSFP_VEST3_rankscore']
                    variant_obj.MetaSVM_score = variant['dbNSFP']['dbNSFP_MetaSVM_score']
                    variant_obj.MetaSVM_rankscore = variant['dbNSFP']['dbNSFP_MetaSVM_rankscore']
                    variant_obj.MetaSVM_pred = variant['dbNSFP']['dbNSFP_MetaSVM_pred']
                    variant_obj.MetaLR_score = variant['dbNSFP']['dbNSFP_MetaLR_score']
                    variant_obj.MetaLR_rankscore = variant['dbNSFP']['dbNSFP_MetaLR_rankscore']
                    variant_obj.MetaLR_pred = variant['dbNSFP']['dbNSFP_MetaLR_pred']
                    variant_obj.Reliability_index = variant['dbNSFP']['dbNSFP_Reliability_index']
                    variant_obj.CADD_raw = variant['dbNSFP']['dbNSFP_CADD_raw']
                    variant_obj.CADD_raw_rankscore = variant['dbNSFP']['dbNSFP_CADD_raw_rankscore']
                    variant_obj.CADD_phred = variant['dbNSFP']['dbNSFP_CADD_phred']
                    variant_obj.DANN_score = variant['dbNSFP']['dbNSFP_DANN_score']
                    variant_obj.DANN_rankscore = variant['dbNSFP']['dbNSFP_DANN_rankscore']
                    variant_obj.fathmm_MKL_coding_score = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_score']
                    variant_obj.fathmm_MKL_coding_rankscore = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_rankscore']
                    variant_obj.fathmm_MKL_coding_pred = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_pred']
                    variant_obj.fathmm_MKL_coding_group = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_group']
                    variant_obj.Eigen_raw = variant['dbNSFP']['dbNSFP_Eigen-raw']
                    variant_obj.Eigen_phred = variant['dbNSFP']['dbNSFP_Eigen-phred']
                    # variant_obj.Eigen_raw_rankscore = variant['dbNSFP']['dbNSFP_Eigen-raw_rankscore']
                    variant_obj.Eigen_PC_raw = variant['dbNSFP']['dbNSFP_Eigen-PC-raw']
                    variant_obj.Eigen_PC_raw_rankscore = variant['dbNSFP']['dbNSFP_Eigen-PC-raw_rankscore']
                    variant_obj.GenoCanyon_score = variant['dbNSFP']['dbNSFP_GenoCanyon_score']
                    variant_obj.GenoCanyon_score_rankscore = variant['dbNSFP']['dbNSFP_GenoCanyon_score_rankscore']
                    variant_obj.integrated_fitCons_score = variant['dbNSFP']['dbNSFP_integrated_fitCons_score']
                    variant_obj.integrated_fitCons_rankscore = variant['dbNSFP']['dbNSFP_integrated_fitCons_rankscore']
                    variant_obj.integrated_confidence_value = variant['dbNSFP']['dbNSFP_integrated_confidence_value']
                    variant_obj.GM12878_fitCons_score = variant['dbNSFP']['dbNSFP_GM12878_fitCons_score']
                    variant_obj.GM12878_fitCons_rankscore = variant['dbNSFP']['dbNSFP_GM12878_fitCons_rankscore']
                    variant_obj.GM12878_confidence_value = variant['dbNSFP']['dbNSFP_GM12878_confidence_value']
                    variant_obj.H1_hESC_fitCons_score = variant['dbNSFP']['dbNSFP_H1-hESC_fitCons_score']
                    variant_obj.H1_hESC_fitCons_rankscore = variant['dbNSFP']['dbNSFP_H1-hESC_fitCons_rankscore']
                    variant_obj.H1_hESC_confidence_value = variant['dbNSFP']['dbNSFP_H1-hESC_confidence_value']
                    variant_obj.HUVEC_fitCons_score = variant['dbNSFP']['dbNSFP_HUVEC_fitCons_score']
                    variant_obj.HUVEC_fitCons_rankscore = variant['dbNSFP']['dbNSFP_HUVEC_fitCons_rankscore']
                    # variant_obj.HUVEC_confidence_value = variant['dbNSFP']['dbNSFP_HUVEC_confidence_value']
                    # variant_obj.GERP_NR = variant['dbNSFP']['dbNSFP_GERP++_NR']
                    # variant_obj.GERP_RS = variant['dbNSFP']['dbNSFP_GERP++_RS']
                    # variant_obj.GERP_RS_rankscore = variant['dbNSFP']['dbNSFP_GERP++_RS_rankscore']
                    # variant_obj.phyloP100way_vertebrate = variant['dbNSFP']['dbNSFP_phyloP100way_vertebrate']
                    # variant_obj.phyloP100way_vertebrate_rankscore = variant['dbNSFP']['dbNSFP_phyloP100way_vertebrate_rankscore']
                    # variant_obj.phyloP20way_mammalian = variant['dbNSFP']['dbNSFP_phyloP20way_mammalian']
                    # variant_obj.phyloP20way_mammalian_rankscore = variant['dbNSFP']['dbNSFP_phyloP20way_mammalian_rankscore']
                    # variant_obj.phastCons100way_vertebrate = variant['dbNSFP']['dbNSFP_phastCons100way_vertebrate']
                    # variant_obj.phastCons100way_vertebrate_rankscore = variant['dbNSFP']['dbNSFP_phastCons100way_vertebrate_rankscore']
                    # variant_obj.phastCons20way_mammalian = variant['dbNSFP']['dbNSFP_phastCons20way_mammalian']
                    # variant_obj.phastCons20way_mammalian_rankscore = variant['dbNSFP']['dbNSFP_phastCons20way_mammalian_rankscore']
                    # variant_obj.SiPhy_29way_pi = variant['dbNSFP']['dbNSFP_SiPhy_29way_pi']
                    # variant_obj.SiPhy_29way_logOdds = variant['dbNSFP']['dbNSFP_SiPhy_29way_logOdds']
                    # variant_obj.SiPhy_29way_logOdds_rankscore = variant['dbNSFP']['dbNSFP_SiPhy_29way_logOdds_rankscore']
                    variant_obj.clinvar_rs = variant['dbNSFP']['dbNSFP_clinvar_rs']
                    variant_obj.clinvar_clnsig = variant['dbNSFP']['dbNSFP_clinvar_clnsig']
                    variant_obj.clinvar_trait = variant['dbNSFP']['dbNSFP_clinvar_trait']
                    variant_obj.clinvar_golden_stars = variant['dbNSFP']['dbNSFP_clinvar_golden_stars']
                    variant_obj.mcap_score = variant['mcap']
                    variant_obj.mcap_rankscore = variant['dbNSFP']['dbNSFP_M-CAP_rankscore']
                    variant_obj.mcap_pred = variant['dbNSFP']['dbNSFP_M-CAP_pred']
                    variant_obj.revel_score = variant['dbNSFP']['dbNSFP_REVEL_score']

                variants.append(variant_obj)
                # print 'query', variant_obj.query
                # print(variant['chr'], variant['pos'])
                # variant_obj.save()
    Variant.objects.bulk_create(variants)

    stop = datetime.datetime.now()
    elapsed = stop - start

    individual.insertion_time = elapsed


    individual.status = 'populated'
    individual.n_lines = count2
    individual.save()


    message = """
            The individual %s was inserted to the database with success!
            Now you can check the variants on the link: \n
            http://mendelmd.org/individuals/view/%s
                """ % (individual.name, individual.id)

    if individual.user:
        send_mail('[Mendel,MD] Individual Populated!', message, 'raonyguimaraes@gmail.com',
              ['raonyguimaraes@gmail.com', individual.user.email], fail_silently=False)
    else:
        send_mail('[Mendel,MD] Individual Populated!', message, 'raonyguimaraes@gmail.com',
              ['raonyguimaraes@gmail.com'], fail_silently=False)
    print('Individual %s Populated!' % (individual.id))

    os.system('echo "Individual %s Populated! %s"' % (individual.name, individual.insertion_time))

    command = 'rm -rf %s/ann_sample' % (filepath)
    os.system(command)

    # Find_Medical_Conditions_and_Medicines.delay(individual.id)



__author__ = 'jpablo'
from django.db import connection, transaction
from django.db.models.fields import  NOT_PROVIDED

def bulk_insert(object_list, show_sql = False):
    """
    Generate the sql code for bulk insertion
    @param object_list: Django model objects
    """
    if not len(object_list):
        return
    Model = type(object_list[0])
    table_name = Model._meta.db_table
    fields_names = [ f.attname for f in Model._meta.fields if f.name != "id" ]
    sql = "insert into " + table_name + ' (' + ','.join(fields_names) + ') values \n'
    defaults = dict([(f.attname, f.default if f.default is not NOT_PROVIDED else "NULL") for f in Model._meta.fields])
    auto_now_add = [f.attname for f in Model._meta.fields if getattr(f, "auto_now_add", False)]

    def get_values(ob, fields):
        ret = []
        for field in fields:
            val = getattr(ob, field)
            if val is None:
                val = defaults[field]
            if field in auto_now_add:
                val = date.today().strftime("%Y-%m-%d")
            ret.append(str(val))
        return ret

    lines = []
    for ob in object_list:
        line = '("' + '","'.join(get_values(ob, fields_names)) + '")'
        line = line.replace('"NULL"', 'NULL')
        line = line.replace('"False"', 'False')
        line = line.replace('"True"', 'False')
        lines.append(line)
    sql += ',\n'.join(lines) + ";"
#    genesfile = open('sqlgenes', 'w')
    if show_sql:
        print(sql)
        return
    cursor = connection.cursor()
#    genesfile.writelines(sql)
    cursor.execute(sql)
    
    transaction.commit_unless_managed()


#!/usr/bin/env python
from setuptools import setup, find_packages

setup(name='mendelmd',
      version='1.0',
      packages=find_packages(),
      scripts=['manage.py', 'bin/mendelmd'],
      # entry_points={  # Optional
      #         'console_scripts': [
      #             'mendelmd=mendelmd.main:main',
      #         ],
      # },
      )



from django.shortcuts import render

from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from collections import OrderedDict as SortedDict
from variants.models import *

import pickle
import json

@login_required
def index(request):
    variants = Variant.objects.all()[:10]
    context = {'variants':variants}
    return render(request, 'variants/index.html', context)

@login_required
def view(request, variant_id):
    variant = get_object_or_404(Variant, pk=variant_id)

    variant.info = json.loads(variant.info)

    new_dict = SortedDict()
    key_list = list(variant.info.keys())
    key_list.sort()
    for key in key_list:
        new_dict[key] = variant.info[key]

    variant.info = new_dict
    print(type(variant.info))

    return render(request, 'variants/view.html', {'variant': variant})



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('files', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('manifest', django.contrib.postgres.fields.jsonb.JSONField()),
                ('status', models.CharField(max_length=30)),
                ('action', models.CharField(max_length=30)),
                ('md5', models.TextField(blank=True, null=True)),
                ('output', models.TextField(blank=True, null=True)),
                ('started', models.DateTimeField(blank=True, null=True)),
                ('finished', models.DateTimeField(blank=True, null=True)),
                ('timetaken', models.DateTimeField(blank=True, null=True)),
                ('total_cost', models.DecimalField(blank=True, decimal_places=10, max_digits=10, null=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(blank=True, null=True)),
                ('files', models.ManyToManyField(to='files.File')),
                ('user', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
from files.models import File
from settings.models import S3Credential
import boto3
import os
from django.conf import settings
import time
from django.core.exceptions import ObjectDoesNotExist

class Command(BaseCommand):
    help = 'Import Files'

    def handle(self, *args, **options):
        print('Hello World Import Files')
        start_time = time.time()

        print('Import Files')
        # File.objects.all().delete()
        file_list = open('%s/data/files/all_files.txt' % (settings.BASE_DIR))
        file_types = {}
        files = {}
        exclude_paths = []
        s3credentials = S3Credential.objects.all()
        for s3credential in s3credentials:
            # print()
            # print(s3credential.exclude_paths.splitlines())
            for item in s3credential.exclude_paths.splitlines():
                exclude_paths.append(item)

        n_files = 0
        for line in file_list:
            row = line.strip().split('\t')
            file = {
                'date': row[0],
                'size': row[1],
                'bucket': row[2],
                'location': row[3],
            }
            if not file['location'].startswith(tuple(exclude_paths)):
                full_path = 's3://' + file['bucket'] + '/' + file['location']
                n_files += 1
                file_name, file_extension = os.path.splitext(file['location'])
                if file_extension == '.gz':
                    file_name, file_extension = os.path.splitext(file_name)
                file['extension'] = file_extension
                if file_extension not in file_types:
                    file_types[file_extension] = []
                file_types[file_extension].append(full_path)
                files[full_path] = file
        print('Summary')
        print('Number of files: {}'.format(n_files))
        print('Number of file types: {}'.format(len(file_types)))
        extensions = ['.fastq', '.bam', '.vcf']
        file_objs = []
        for extension in extensions:
            for file in file_types[extension]:
                # print(file)
                file_name, file_extension = os.path.splitext(file)
                if file_extension == '.gz':
                    file_name, file_extension = os.path.splitext(file_name)
                basename = os.path.basename(file_name)
                try:
                    test = File.objects.get(location=file)
                except ObjectDoesNotExist:
                    file_obj = File(
                        name=basename,
                        size=files[file]['size'],
                        last_modified=str(files[file]['date']),
                        extension=file_extension.replace('.', ''),
                        location=file,
                    )
                    #print(file_obj,files[file])
                    file_objs.append(file_obj)
        File.objects.bulk_create(file_objs)

        elapsed_time = time.time() - start_time
        print('Importing Files Took {}'.format(elapsed_time))


from django.core.management.base import BaseCommand, CommandError

from django.db import connection
from databases.models import HGMD
        
class Command(BaseCommand):
    # args = '<poll_id poll_id ...>'
    help = 'Import data from hgmd'

    def handle(self, *args, **options):

        hgmd = open('/projects/hgmd/HGMD_PRO_2017.3_hg19.vcf')
        hgmd_list = []
        for line in hgmd:

            # print(line)
            if not line.startswith('#'):
                row = line.split('\t')
                index = '{}-{}-{}-{}'.format(row[0], row[1], row[3], row[4])
                hgmd_entry = HGMD()

                info = row[7].strip().split(';')
                info_dict = {}
                for tag in info:
                    item = tag.split('=')
                    info_dict[item[0]] = item[1]

                tags = ['CLASS', 'MUT', 'GENE', 'STRAND', 'DNA', 'PROT', 'DB', 'PHEN']
                for item in tags:
                    if item not in info_dict:
                        info_dict[item] = None
                # print(info_dict)

                hgmd_entry.index = index
                hgmd_entry.chrom = row[0]
                hgmd_entry.pos = row[1]
                hgmd_entry.rsid = row[2]
                hgmd_entry.ref = row[3]
                hgmd_entry.alt = row[4]
                hgmd_entry.qual = row[5]
                hgmd_entry.filter = row[6]
                hgmd_entry.mutclass = info_dict['CLASS']
                hgmd_entry.mut = info_dict['MUT']
                hgmd_entry.gene = info_dict['GENE']
                hgmd_entry.strand = info_dict['STRAND']
                hgmd_entry.dna = info_dict['DNA']
                hgmd_entry.prot = info_dict['PROT']
                hgmd_entry.db = info_dict['DB']
                hgmd_entry.phen = info_dict['PHEN']
                hgmd_list.append(hgmd_entry)

        HGMD.objects.bulk_create(hgmd_list)



from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^$', views.index, name='tasks-index'),
    url(r'^delete/(?P<pk>[0-9]+)/$', views.TaskDelete.as_view(), name='tasks-delete'),
    # url(r'^create/$', views.create, name='projects-create'),
    url(r'^view/(?P<pk>[0-9]+)/$', views.TaskDetail.as_view(), name='tasks-view'),
    url(r'^run/(?P<task_id>[0-9]+)/$', views.run_task, name='tasks-run'),
    # url(r'^update/(?P<pk>[0-9]+)/$', views.ProjectUpdate.as_view(), name='projects-update'),
    
    # url(r'^import_files/(?P<project_id>[0-9]+)/$', views.import_files, name='projects-import-files'),
    # url(r'^import_project_files/(?P<project_id>[0-9]+)/$', views.import_project_files, name='import_project_files'),
    url(r'^bulk_action/$', views.bulk_action, name='tasks-bulk-action'),
]



"""
Django settings for mendelmd project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""
import sys

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# sys.path.insert(0, os.path.join(BASE_DIR, "apps"))

# APPS_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'apps')

# sys.path.append(APPS_DIR)

# URL_PREFIX = "/mendelmd"
# FORCE_SCRIPT_NAME = '/mendelmd'

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '*efl#$$!@93)8397wwf8hy387"&^%3&ad8h7d2w-yus5mzcx&@'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'django.contrib.humanize',

    #external libs
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    
    'allauth.socialaccount.providers.google',
    'crispy_forms',
    'django_select2',

    # 'djcelery',
    'celery',
    'django_celery_results',
    # 'kombu.transport.django',
    # 'django_celery_results',
    # 'django_celery_beat',

    #private apps

    'dashboard',
    'individuals',
    'variants',
    'diseases',
    'genes',
    'pagination',
    'cases',
    'filter_analysis',
    'pathway_analysis',
    'stats',
    'databases',
    'projects',
    'files',
    'samples',
    'upload',
    'settings',
    'tasks',
    'workers',
    'analyses',
    'formtools',
    'mapps',
    'django_gravatar',    
    #'storages',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]



ROOT_URLCONF = 'mendelmd.urls'

WSGI_APPLICATION = 'mendelmd.wsgi.application'

#DATABASES = {
#    'default': {
#        'ENGINE': 'django.db.backends.postgresql_psycopg2',
#        'NAME': 'mendelmd',
#    }
#}
DATABASES = {
    'default': {
         'ENGINE': 'django.db.backends.sqlite3',
         'NAME': 'mendelmd.db',
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'
# TIME_ZONE = 'America/Sao_Paulo'

USE_I18N = False

USE_L10N = False

USE_TZ = False


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'

#STATIC_ROOT = os.path.join(BASE_DIR,'static/')

#STATIC_ROOT = '/var/www/static/'

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static"),
]


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            # insert your TEMPLATE_DIRS here
            os.path.join(BASE_DIR, "templates"),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.request',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                # Required by allauth template tags
                
            ],

        },
    },
]



AUTHENTICATION_BACKENDS = [
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend",
]

SITE_ID = 1

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

LOGIN_REDIRECT_URL = 'dashboard'

CRISPY_TEMPLATE_PACK = 'bootstrap3'

#this prevents crash when loading filter_analysis forms.py
# DEBUG_TOOLBAR_PATCH_SETTINGS = True

INTERNAL_IPS = ['127.0.0.1']

# new celery 4 config
CELERY_RESULT_BACKEND = 'django-db'
CELERY_RESULT_BACKEND = 'django-cache'

CELERY_IMPORTS = ('analyses.tasks','tasks.tasks','workers.tasks','individuals.tasks')

CELERYBEAT_SCHEDULE = {
    'check_queue': {
        'task': 'workers.tasks.check_queue',
        'schedule': 30.0,
    },
}

ALLOWED_HOSTS = ['localhost', '127.0.0.1', '*']

try:
    from .local_settings import *
except ImportError:
    pass

FILE_UPLOAD_PERMISSIONS = 0o0777
from datetime import timedelta

DATA_UPLOAD_MAX_NUMBER_FIELDS = 100000

# ALL AUTH
ACCOUNT_EMAIL_REQUIRED=True
ACCOUNT_AUTHENTICATION_METHOD="username_email"
ACCOUNT_SESSION_REMEMBER=True
# ACCOUNT_UNIQUE_EMAIL=False
SOCIALACCOUNT_AUTO_SIGNUP=True
SOCIALACCOUNT_QUERY_EMAIL=True



from django.contrib.postgres.fields import JSONField
from django.db import models
from django.contrib.auth.models import User
from files.models import File
# from analyses.models import Analysis

class Task(models.Model):

    user = models.ForeignKey(User, editable=False, null=True, blank=True, on_delete=models.CASCADE)
    name = models.CharField(max_length=30)
    manifest = JSONField()
    
    status = models.CharField(max_length=30)
    action = models.CharField(max_length=30)
    md5 = models.TextField(null=True, blank=True)

    output = models.TextField(null=True, blank=True)
    
    started = models.DateTimeField(null=True, blank=True)
    finished = models.DateTimeField(null=True, blank=True)
    timetaken = models.DateTimeField(null=True, blank=True)
    
    total_cost = models.DecimalField(null=True, blank=True, max_digits=10, decimal_places=10)

    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(null=True, blank=True)
    files = models.ManyToManyField(File)

    def __str__(self):
        return self.name



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Dbnfsp',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('chr', models.TextField(blank=True, null=True)),
                ('pos_1_based', models.TextField(blank=True, null=True)),
                ('ref', models.TextField(blank=True, null=True)),
                ('alt', models.TextField(blank=True, null=True)),
                ('aaref', models.TextField(blank=True, null=True)),
                ('aaalt', models.TextField(blank=True, null=True)),
                ('rs_dbSNP150', models.TextField(blank=True, null=True)),
                ('hg19_chr', models.TextField(blank=True, null=True)),
                ('hg19_pos_1_based', models.TextField(blank=True, null=True)),
                ('hg18_chr', models.TextField(blank=True, null=True)),
                ('hg18_pos_1_based', models.TextField(blank=True, null=True)),
                ('genename', models.TextField(blank=True, null=True)),
                ('cds_strand', models.TextField(blank=True, null=True)),
                ('refcodon', models.TextField(blank=True, null=True)),
                ('codonpos', models.TextField(blank=True, null=True)),
                ('codon_degeneracy', models.TextField(blank=True, null=True)),
                ('Ancestral_allele', models.TextField(blank=True, null=True)),
                ('AltaiNeandertal', models.TextField(blank=True, null=True)),
                ('Denisova', models.TextField(blank=True, null=True)),
                ('Ensembl_geneid', models.TextField(blank=True, null=True)),
                ('Ensembl_transcriptid', models.TextField(blank=True, null=True)),
                ('Ensembl_proteinid', models.TextField(blank=True, null=True)),
                ('aapos', models.TextField(blank=True, null=True)),
                ('SIFT_score', models.TextField(blank=True, null=True)),
                ('SIFT_converted_rankscore', models.TextField(blank=True, null=True)),
                ('SIFT_pred', models.TextField(blank=True, null=True)),
                ('Uniprot_acc_Polyphen2', models.TextField(blank=True, null=True)),
                ('Uniprot_id_Polyphen2', models.TextField(blank=True, null=True)),
                ('Uniprot_aapos_Polyphen2', models.TextField(blank=True, null=True)),
                ('Polyphen2_HDIV_score', models.TextField(blank=True, null=True)),
                ('Polyphen2_HDIV_rankscore', models.TextField(blank=True, null=True)),
                ('Polyphen2_HDIV_pred', models.TextField(blank=True, null=True)),
                ('Polyphen2_HVAR_score', models.TextField(blank=True, null=True)),
                ('Polyphen2_HVAR_rankscore', models.TextField(blank=True, null=True)),
                ('Polyphen2_HVAR_pred', models.TextField(blank=True, null=True)),
                ('LRT_score', models.TextField(blank=True, null=True)),
                ('LRT_converted_rankscore', models.TextField(blank=True, null=True)),
                ('LRT_pred', models.TextField(blank=True, null=True)),
                ('LRT_Omega', models.TextField(blank=True, null=True)),
                ('MutationTaster_score', models.TextField(blank=True, null=True)),
                ('MutationTaster_converted_rankscore', models.TextField(blank=True, null=True)),
                ('MutationTaster_pred', models.TextField(blank=True, null=True)),
                ('MutationTaster_model', models.TextField(blank=True, null=True)),
                ('MutationTaster_AAE', models.TextField(blank=True, null=True)),
                ('MutationAssessor_UniprotID', models.TextField(blank=True, null=True)),
                ('MutationAssessor_variant', models.TextField(blank=True, null=True)),
                ('MutationAssessor_score', models.TextField(blank=True, null=True)),
                ('MutationAssessor_score_rankscore', models.TextField(blank=True, null=True)),
                ('MutationAssessor_pred', models.TextField(blank=True, null=True)),
                ('FATHMM_score', models.TextField(blank=True, null=True)),
                ('FATHMM_converted_rankscore', models.TextField(blank=True, null=True)),
                ('FATHMM_pred', models.TextField(blank=True, null=True)),
                ('PROVEAN_score', models.TextField(blank=True, null=True)),
                ('PROVEAN_converted_rankscore', models.TextField(blank=True, null=True)),
                ('PROVEAN_pred', models.TextField(blank=True, null=True)),
                ('Transcript_id_VEST3', models.TextField(blank=True, null=True)),
                ('Transcript_var_VEST3', models.TextField(blank=True, null=True)),
                ('VEST3_score', models.TextField(blank=True, null=True)),
                ('VEST3_rankscore', models.TextField(blank=True, null=True)),
                ('MetaSVM_score', models.TextField(blank=True, null=True)),
                ('MetaSVM_rankscore', models.TextField(blank=True, null=True)),
                ('MetaSVM_pred', models.TextField(blank=True, null=True)),
                ('MetaLR_score', models.TextField(blank=True, null=True)),
                ('MetaLR_rankscore', models.TextField(blank=True, null=True)),
                ('MetaLR_pred', models.TextField(blank=True, null=True)),
                ('Reliability_index', models.TextField(blank=True, null=True)),
                ('M_CAP_score', models.TextField(blank=True, null=True)),
                ('M_CAP_rankscore', models.TextField(blank=True, null=True)),
                ('M_CAP_pred', models.TextField(blank=True, null=True)),
                ('REVEL_score', models.TextField(blank=True, null=True)),
                ('REVEL_rankscore', models.TextField(blank=True, null=True)),
                ('MutPred_score', models.TextField(blank=True, null=True)),
                ('MutPred_rankscore', models.TextField(blank=True, null=True)),
                ('MutPred_protID', models.TextField(blank=True, null=True)),
                ('MutPred_AAchange', models.TextField(blank=True, null=True)),
                ('MutPred_Top5features', models.TextField(blank=True, null=True)),
                ('CADD_raw', models.TextField(blank=True, null=True)),
                ('CADD_raw_rankscore', models.TextField(blank=True, null=True)),
                ('CADD_phred', models.TextField(blank=True, null=True)),
                ('DANN_score', models.TextField(blank=True, null=True)),
                ('DANN_rankscore', models.TextField(blank=True, null=True)),
                ('fathmm_MKL_coding_score', models.TextField(blank=True, null=True)),
                ('fathmm_MKL_coding_rankscore', models.TextField(blank=True, null=True)),
                ('fathmm_MKL_coding_pred', models.TextField(blank=True, null=True)),
                ('fathmm_MKL_coding_group', models.TextField(blank=True, null=True)),
                ('Eigen_coding_or_noncoding', models.TextField(blank=True, null=True)),
                ('Eigen_raw', models.TextField(blank=True, null=True)),
                ('Eigen_phred', models.TextField(blank=True, null=True)),
                ('Eigen_PC_raw', models.TextField(blank=True, null=True)),
                ('Eigen_PC_phred', models.TextField(blank=True, null=True)),
                ('Eigen_PC_raw_rankscore', models.TextField(blank=True, null=True)),
                ('GenoCanyon_score', models.TextField(blank=True, null=True)),
                ('GenoCanyon_score_rankscore', models.TextField(blank=True, null=True)),
                ('integrated_fitCons_score', models.TextField(blank=True, null=True)),
                ('integrated_fitCons_score_rankscore', models.TextField(blank=True, null=True)),
                ('integrated_confidence_value', models.TextField(blank=True, null=True)),
                ('GM12878_fitCons_score', models.TextField(blank=True, null=True)),
                ('GM12878_fitCons_score_rankscore', models.TextField(blank=True, null=True)),
                ('GM12878_confidence_value', models.TextField(blank=True, null=True)),
                ('H1_hESC_fitCons_score', models.TextField(blank=True, null=True)),
                ('H1_hESC_fitCons_score_rankscore', models.TextField(blank=True, null=True)),
                ('H1_hESC_confidence_value', models.TextField(blank=True, null=True)),
                ('HUVEC_fitCons_score', models.TextField(blank=True, null=True)),
                ('HUVEC_fitCons_score_rankscore', models.TextField(blank=True, null=True)),
                ('HUVEC_confidence_value', models.TextField(blank=True, null=True)),
                ('GERP_NR', models.TextField(blank=True, null=True)),
                ('GERP_RS', models.TextField(blank=True, null=True)),
                ('GERP_RS_rankscore', models.TextField(blank=True, null=True)),
                ('phyloP100way_vertebrate', models.TextField(blank=True, null=True)),
                ('phyloP100way_vertebrate_rankscore', models.TextField(blank=True, null=True)),
                ('phyloP20way_mammalian', models.TextField(blank=True, null=True)),
                ('phyloP20way_mammalian_rankscore', models.TextField(blank=True, null=True)),
                ('phastCons100way_vertebrate', models.TextField(blank=True, null=True)),
                ('phastCons100way_vertebrate_rankscore', models.TextField(blank=True, null=True)),
                ('phastCons20way_mammalian', models.TextField(blank=True, null=True)),
                ('phastCons20way_mammalian_rankscore', models.TextField(blank=True, null=True)),
                ('SiPhy_29way_pi', models.TextField(blank=True, null=True)),
                ('SiPhy_29way_logOdds', models.TextField(blank=True, null=True)),
                ('SiPhy_29way_logOdds_rankscore', models.TextField(blank=True, null=True)),
                ('Gp3_AC_1k', models.TextField(blank=True, null=True)),
                ('Gp3_AF_1k', models.TextField(blank=True, null=True)),
                ('Gp3_AFR_AC_1k', models.TextField(blank=True, null=True)),
                ('Gp3_AFR_AF_1k', models.TextField(blank=True, null=True)),
                ('Gp3_EUR_AC_1k', models.TextField(blank=True, null=True)),
                ('Gp3_EUR_AF_1k', models.TextField(blank=True, null=True)),
                ('Gp3_AMR_AC_1k', models.TextField(blank=True, null=True)),
                ('Gp3_AMR_AF_1k', models.TextField(blank=True, null=True)),
                ('Gp3_EAS_AC_1k', models.TextField(blank=True, null=True)),
                ('Gp3_EAS_AF_1k', models.TextField(blank=True, null=True)),
                ('Gp3_SAS_AC_1k', models.TextField(blank=True, null=True)),
                ('Gp3_SAS_AF_1k', models.TextField(blank=True, null=True)),
                ('TWINSUK_AC', models.TextField(blank=True, null=True)),
                ('TWINSUK_AF', models.TextField(blank=True, null=True)),
                ('ALSPAC_AC', models.TextField(blank=True, null=True)),
                ('ALSPAC_AF', models.TextField(blank=True, null=True)),
                ('ESP6500_AA_AC', models.TextField(blank=True, null=True)),
                ('ESP6500_AA_AF', models.TextField(blank=True, null=True)),
                ('ESP6500_EA_AC', models.TextField(blank=True, null=True)),
                ('ESP6500_EA_AF', models.TextField(blank=True, null=True)),
                ('ExAC_AC', models.TextField(blank=True, null=True)),
                ('ExAC_AF', models.TextField(blank=True, null=True)),
                ('ExAC_Adj_AC', models.TextField(blank=True, null=True)),
                ('ExAC_Adj_AF', models.TextField(blank=True, null=True)),
                ('ExAC_AFR_AC', models.TextField(blank=True, null=True)),
                ('ExAC_AFR_AF', models.TextField(blank=True, null=True)),
                ('ExAC_AMR_AC', models.TextField(blank=True, null=True)),
                ('ExAC_AMR_AF', models.TextField(blank=True, null=True)),
                ('ExAC_EAS_AC', models.TextField(blank=True, null=True)),
                ('ExAC_EAS_AF', models.TextField(blank=True, null=True)),
                ('ExAC_FIN_AC', models.TextField(blank=True, null=True)),
                ('ExAC_FIN_AF', models.TextField(blank=True, null=True)),
                ('ExAC_NFE_AC', models.TextField(blank=True, null=True)),
                ('ExAC_NFE_AF', models.TextField(blank=True, null=True)),
                ('ExAC_SAS_AC', models.TextField(blank=True, null=True)),
                ('ExAC_SAS_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_Adj_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_Adj_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_AFR_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_AFR_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_AMR_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_AMR_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_EAS_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_EAS_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_FIN_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_FIN_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_NFE_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_NFE_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_SAS_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonTCGA_SAS_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_Adj_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_Adj_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_AFR_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_AFR_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_AMR_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_AMR_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_EAS_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_EAS_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_FIN_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_FIN_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_NFE_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_NFE_AF', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_SAS_AC', models.TextField(blank=True, null=True)),
                ('ExAC_nonpsych_SAS_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AFR_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AFR_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AFR_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AMR_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AMR_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_AMR_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_ASJ_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_ASJ_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_ASJ_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_EAS_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_EAS_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_EAS_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_FIN_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_FIN_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_FIN_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_NFE_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_NFE_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_NFE_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_SAS_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_SAS_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_SAS_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_OTH_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_OTH_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_exomes_OTH_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AFR_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AFR_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AFR_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AMR_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AMR_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_AMR_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_ASJ_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_ASJ_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_ASJ_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_EAS_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_EAS_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_EAS_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_FIN_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_FIN_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_FIN_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_NFE_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_NFE_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_NFE_AF', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_OTH_AC', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_OTH_AN', models.TextField(blank=True, null=True)),
                ('gnomAD_genomes_OTH_AF', models.TextField(blank=True, null=True)),
                ('clinvar_rs', models.TextField(blank=True, null=True)),
                ('clinvar_clnsig', models.TextField(blank=True, null=True)),
                ('clinvar_trait', models.TextField(blank=True, null=True)),
                ('clinvar_golden_stars', models.TextField(blank=True, null=True)),
                ('Interpro_domain', models.TextField(blank=True, null=True)),
                ('GTEx_V6p_gene', models.TextField(blank=True, null=True)),
                ('GTEx_V6p_tissue', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Genome1kGenotype',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('genotype', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Genome1kSample',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Genome1kSampleVariant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('genotype', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='databases.Genome1kGenotype')),
                ('sample', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='databases.Genome1kSample')),
            ],
        ),
        migrations.CreateModel(
            name='Genome1kVariant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pos_index', models.TextField(db_index=True)),
                ('chrom', models.TextField(blank=True, db_index=True, null=True)),
                ('pos', models.TextField(blank=True, db_index=True, null=True)),
                ('rsid', models.TextField(blank=True, null=True)),
                ('ref', models.TextField(blank=True, db_index=True, null=True)),
                ('alt', models.TextField(blank=True, db_index=True, null=True)),
                ('qual', models.TextField(blank=True, null=True)),
                ('filter', models.TextField(blank=True, null=True)),
                ('info', models.TextField(blank=True, null=True)),
                ('format', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Genome1kVariantIndex',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('index', models.TextField()),
                ('variant', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='databases.Genome1kVariant')),
            ],
        ),
        migrations.CreateModel(
            name='HGMD',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('index', models.TextField(db_index=True)),
                ('chrom', models.TextField(blank=True, db_index=True, null=True)),
                ('pos', models.TextField(blank=True, db_index=True, null=True)),
                ('rsid', models.TextField(blank=True, null=True)),
                ('ref', models.TextField(blank=True, db_index=True, null=True)),
                ('alt', models.TextField(blank=True, db_index=True, null=True)),
                ('qual', models.TextField(blank=True, null=True)),
                ('filter', models.TextField(blank=True, null=True)),
                ('mutclass', models.TextField(blank=True, null=True)),
                ('mut', models.TextField(blank=True, null=True)),
                ('gene', models.TextField(blank=True, null=True)),
                ('strand', models.TextField(blank=True, null=True)),
                ('dna', models.TextField(blank=True, null=True)),
                ('prot', models.TextField(blank=True, null=True)),
                ('db', models.TextField(blank=True, null=True)),
                ('phen', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='VariSNP',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dbsnp_id', models.CharField(max_length=255)),
            ],
        ),
        migrations.AddField(
            model_name='genome1ksamplevariant',
            name='variant',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='databases.Genome1kVariant'),
        ),
    ]



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('params', django.contrib.postgres.fields.jsonb.JSONField()),
            ],
        ),
        migrations.CreateModel(
            name='Provider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('type', models.CharField(max_length=255)),
                ('config', django.contrib.postgres.fields.jsonb.JSONField()),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='S3Credential',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('access_key', models.CharField(max_length=255)),
                ('secret_key', models.CharField(max_length=255)),
                ('buckets', models.TextField(blank=True, null=True)),
                ('exclude_paths', models.TextField(blank=True, null=True)),
                ('exclude_files', models.TextField(blank=True, null=True)),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Subnet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('config', django.contrib.postgres.fields.jsonb.JSONField()),
                ('count', models.IntegerField(blank=True, null=True)),
                ('provider', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='settings.Provider')),
            ],
        ),
    ]
